-- xxx: needs to be converted to lua 5.2

local bdc = require("_bdh")
local lfs = require("lfs")

-- converts a directory of level files named "###.level" to the internal
-- format used by Family Picross. "###" represents a number from 001 to 256.
-- Meant to be used with Normal Mode puzzles.

-- Usage:
-- 1) lua _convert.lua > puzzles.h
-- 2) Move file to src/ folder and overwrite.
-- 3) Recompile and play.

print("/* puzzle data (generated by Family Picross Mass Level Converter Tool) */")
print()

local t = {
	["0"] = "0000",
	["1"] = "0001",
	["2"] = "0010",
	["3"] = "0011",
	["4"] = "0100",
	["5"] = "0101",
	["6"] = "0110",
	["7"] = "0111",
	["8"] = "1000",
	["9"] = "1001",
	a = "1010",
	b = "1011",
	c = "1100",
	d = "1101",
	e = "1110",
	f = "1111"
};

--- Converts the given number, n, to a string in a binary number format (12
-- becomes "1100").
-- @param n Number to convert.
-- @return String in binary format.
function tobinary(n)
    assert(tonumber(n), "number expected");
    return (("%x"):format(n):gsub("%w", t):gsub("^0*", ""));
end

local diffToNum = {
	easy	= 0,
	medium	= 1,
	hard	= 2,
	expert	= 3,
	master	= 4
	-- everything else is invalid.
}

-- most of this is taken from the levelgen.lua tool
local function ReadLevelFile(_level)
	local lineCounter=0
	local rowCounter=1

	local levelData = {}
	levelData.Rows = {}

	for line in _level:lines() do
		if lineCounter == 0 then levelData.Name=line
		elseif lineCounter == 1 then levelData.Size=tonumber(line)
		elseif lineCounter == 2 then levelData.Difficulty=line
		else
			levelData.Rows[rowCounter] = line
			rowCounter = rowCounter + 1
		end

		-- check for long names
		if lineCounter == 0 then
			if string.len(levelData.Name) > 16 then
				print(string.format("WARNING: Level Name '%s' is longer than 16 characters!",levelData.Name))
			end
		end

		-- check for level size
		if lineCounter == 1 then
			if levelData.Size < 5 or levelData.Size > 16 then
				-- error out right now since scrolling levels don't exist yet
				print(string.format("Invalid level size '%d'.",levelData.Size))
				return nil
			end
		end

		-- convert difficulty
		if lineCounter == 2 then
			local diff = string.lower(levelData.Difficulty)
			local diffNum = diffToNum[diff]
			if diffNum then
				levelData.DifficultyNum = diffNum
			else
				print(string.format("Invalid difficulty '%s'. Setting to easy.",diff))
				levelData.DifficultyNum = 0
			end
		end

		lineCounter = lineCounter + 1
	end

	return levelData
end

-- most of this is either taken from levelgen.lua, or created here first and
-- adapted to levelgen.lua later. I should have a re-usable version of this code.
local function ConvertPuzzle(_data,_levelnum)
	local xformPuzzle = {} -- transformed puzzle from human-readable to native

	if _data.Size == 8 then
		-- 8x8; 1 row = 1 byte
		for k,v in pairs(_data.Rows) do
			xformPuzzle[k] = bdc.Bin2Hex(v)
		end

	elseif _data.Size == 16 then
		-- 16x16; 1 row = 2 bytes
		for k,v in pairs(_data.Rows) do
			local top = bdc.Bin2Hex(string.sub(v,1,8))
			local bot = bdc.Bin2Hex(string.sub(v,9,-1))
			xformPuzzle[k] = top..bot
		end

	-- todo: 12x12 uses alternating (8, 4/4, 8) patterns for 2 rows;
	-- this is simpler than most of the patterns below (even though it uses
	-- the same parsing pattern as those).
	elseif _data.Size == 12 then
		-- row 1:	( 1)11111111, ( 2)11110000
		local row1 = _data.Rows[1]
		local row1a, row1b = ""
		row1a = bdc.Bin2Hex(string.sub(row1,1,8))
		row1b = bdc.Bin2Hex(string.sub(row1,9,-1).."0000")

		-- row 2:	( 2)00001111, ( 3)11111111
		local row2 = _data.Rows[2]
		local row2a, row2b = ""
		row2a = bdc.Bin2Hex("0000"..string.sub(row2,1,4))
		row2b = bdc.Bin2Hex(string.sub(row2,5,-1))

		-- row 3:	( 4)11111111, ( 5)11110000
		local row3 = _data.Rows[3]
		local row3a, row3b = ""
		row3a = bdc.Bin2Hex(string.sub(row3,1,8))
		row3b = bdc.Bin2Hex(string.sub(row3,9,-1).."0000")

		-- row 4:	( 5)00001111, ( 6)11111111
		local row4 = _data.Rows[4]
		local row4a, row4b = ""
		row4a = bdc.Bin2Hex("0000"..string.sub(row4,1,4))
		row4b = bdc.Bin2Hex(string.sub(row4,5,-1))

		-- row 5:	( 7)11111111, ( 8)11110000
		local row5 = _data.Rows[5]
		local row5a, row5b = ""
		row5a = bdc.Bin2Hex(string.sub(row5,1,8))
		row5b = bdc.Bin2Hex(string.sub(row5,9,-1).."0000")

		-- row 6:	( 8)00001111, ( 9)11111111
		local row6 = _data.Rows[6]
		local row6a, row6b = ""
		row6a = bdc.Bin2Hex("0000"..string.sub(row6,1,4))
		row6b = bdc.Bin2Hex(string.sub(row6,5,-1))

		-- row 7:	(10)11111111, (11)11110000
		local row7 = _data.Rows[7]
		local row7a, row7b = ""
		row7a = bdc.Bin2Hex(string.sub(row7,1,8))
		row7b = bdc.Bin2Hex(string.sub(row7,9,-1).."0000")

		-- row 8:	(11)00001111, (12)11111111
		local row8 = _data.Rows[8]
		local row8a, row8b = ""
		row8a = bdc.Bin2Hex("0000"..string.sub(row8,1,4))
		row8b = bdc.Bin2Hex(string.sub(row8,5,-1))

		-- row 9:	(13)11111111, (14)11110000
		local row9 = _data.Rows[9]
		local row9a, row9b = ""
		row9a = bdc.Bin2Hex(string.sub(row9,1,8))
		row9b = bdc.Bin2Hex(string.sub(row9,9,-1).."0000")

		-- row 10:	(14)00001111, (15)11111111
		local row10 = _data.Rows[10]
		local row10a, row10b = ""
		row10a = bdc.Bin2Hex("0000"..string.sub(row10,1,4))
		row10b = bdc.Bin2Hex(string.sub(row10,5,-1))

		-- row 11:	(16)11111111, (17)11110000
		local row11 = _data.Rows[11]
		local row11a, row11b = ""
		row11a = bdc.Bin2Hex(string.sub(row11,1,8))
		row11b = bdc.Bin2Hex(string.sub(row11,9,-1).."0000")

		-- row 12:	(17)00001111, (18)11111111
		local row12 = _data.Rows[12]
		local row12a, row12b = ""
		row12a = bdc.Bin2Hex("0000"..string.sub(row12,1,4))
		row12b = bdc.Bin2Hex(string.sub(row12,5,-1))

		-----------------------------------------------------
		-- byte  1 (1a)
		xformPuzzle[1] = row1a
		-- byte  2 (1b,2a)
		xformPuzzle[2] = bdc.BMOr(row1b,row2a)
		-- byte  3 (2b)
		xformPuzzle[3] = row2b
		-- byte  4 (3a)
		xformPuzzle[4] = row3a
		-- byte  5 (3b,4a)
		xformPuzzle[5] = bdc.BMOr(row3b,row4a)
		-- byte  6 (4b)
		xformPuzzle[6] = row4b
		-- byte  7 (5a)
		xformPuzzle[7] = row5a
		-- byte  8 (5b,6a)
		xformPuzzle[8] = bdc.BMOr(row5b,row6a)
		-- byte  9 (6b)
		xformPuzzle[9] = row6b
		-- byte 10 (7a)
		xformPuzzle[10] = row7a
		-- byte 11 (7b,8a)
		xformPuzzle[11] = bdc.BMOr(row7b,row8a)
		-- byte 12 (8b)
		xformPuzzle[12] = row8b
		-- byte 13 (9a)
		xformPuzzle[13] = row9a
		-- byte 14 (9b,10a)
		xformPuzzle[14] = bdc.BMOr(row9b,row10a)
		-- byte 15 (10b)
		xformPuzzle[15] = row10b
		-- byte 16 (11a)
		xformPuzzle[16] = row11a
		-- byte 17 (11b,12a)
		xformPuzzle[17] = bdc.BMOr(row11b,row12a)
		-- byte 18 (12b)
		xformPuzzle[18] = row12b

	else
		-- anything else... hardcoded bullshit ahoy, for now

		-- 5x5:
		if _data.Size == 5 then
			-- row 1: (1)11111000
			local row1 = bdc.Bin2Hex(_data.Rows[1].."000")

			-- row 2: (1)00000111, (2)11000000
			local row2 = _data.Rows[2]
			local row2a, row2b = ""
			row2a = bdc.Bin2Hex("00000"..string.sub(row2,1,3))
			row2b = bdc.Bin2Hex(string.sub(row2,4,-1).."000000")

			-- row 3: (2)00111110
			local row3 = bdc.Bin2Hex("00".._data.Rows[3].."0")

			-- row 4: (2)00000001, (3)11110000
			local row4 = _data.Rows[4]
			local row4a, row4b = ""
			row4a = bdc.Bin2Hex("0000000"..string.sub(row4,1,1))
			row4b = bdc.Bin2Hex(string.sub(row4,2,-1).."0000")

			-- row 5: (3)00001111, (4)10000000
			local row5 = _data.Rows[5]
			local row5a, row5b = ""
			row5a = bdc.Bin2Hex("0000"..string.sub(row5,1,4))
			row5b = bdc.Bin2Hex(string.sub(row5,5,-1).."0000000")
			-----------------------------------------------------
			-- byte 1 (1,2a)
			local byte1 = bdc.BMOr(row1,row2a)
			xformPuzzle[1] = byte1

			-- byte 2 (2b,3,4a)
			local byte2a = bdc.BMOr(row2b,row3)
			byte2a = bdc.BMOr(byte2a,row4a)
			xformPuzzle[2] = byte2a

			-- byte 3 (4b, 5a)
			local byte3 = bdc.BMOr(row4b,row5a)
			xformPuzzle[3] = byte3

			-- byte 4 (5b)
			xformPuzzle[4] = row5b
		----------------------------------------------------------------------------
		-- 6x6
		elseif _data.Size == 6 then
			-- row 1: (1)11111100
			local row1 = bdc.Bin2Hex(_data.Rows[1].."00")

			-- row 2: (1)00000011, (2)11110000
			local row2 = _data.Rows[2]
			local row2a, row2b = ""
			row2a = bdc.Bin2Hex("000000"..string.sub(row2,1,2))
			row2b = bdc.Bin2Hex(string.sub(row2,3,-1).."0000")

			-- row 3: (2)00001111, (3)11000000
			local row3 = _data.Rows[3]
			local row3a, row3b = ""
			row3a = bdc.Bin2Hex("0000"..string.sub(row3,1,4))
			row3b = bdc.Bin2Hex(string.sub(row3,5,-1).."000000")

			-- row 4: (3)00111111
			local row4 = bdc.Bin2Hex("00".._data.Rows[4])

			-- row 5: (4)11111100
			local row5 = bdc.Bin2Hex(_data.Rows[5].."00")

			-- row 6: (4)00000011, (5)11110000
			local row6 = _data.Rows[6]
			local row6a, row6b = ""
			row6a = bdc.Bin2Hex("000000"..string.sub(row6,1,2))
			row6b = bdc.Bin2Hex(string.sub(row6,3,-1).."0000")
			-----------------------------------------------------
			-- byte 1 (1,2a)
			local byte1 = bdc.BMOr(row1,row2a)
			xformPuzzle[1] = byte1

			-- byte 2 (2b,3a)
			local byte2a = bdc.BMOr(row2b,row3a)
			xformPuzzle[2] = byte2a

			-- byte 3 (3b,4)
			local byte3 = bdc.BMOr(row3b,row4)
			xformPuzzle[3] = byte3

			-- byte 4 (5,6a)
			local byte4 = bdc.BMOr(row5,row6a)
			xformPuzzle[4] = byte4

			-- byte 5 (6b)
			xformPuzzle[5] = row6b
		----------------------------------------------------------------------------
		-- 7x7
		elseif _data.Size == 7 then
			-- row 1: (1)11111110
			local row1 = bdc.Bin2Hex(_data.Rows[1].."0")

			-- row 2: (1)00000001, (2)11111100
			local row2 = _data.Rows[2]
			local row2a, row2b = ""
			row2a = bdc.Bin2Hex("0000000"..string.sub(row2,1,1))
			row2b = bdc.Bin2Hex(string.sub(row2,2,-1).."00")

			-- row 3: (2)00000011, (3)11111000
			local row3 = _data.Rows[3]
			local row3a, row3b = ""
			row3a = bdc.Bin2Hex("000000"..string.sub(row3,1,2))
			row3b = bdc.Bin2Hex(string.sub(row3,3,-1).."000")

			-- row 4: (3)00000111, (4)11110000
			local row4 = _data.Rows[4]
			local row4a, row4b = ""
			row4a = bdc.Bin2Hex("00000"..string.sub(row4,1,3))
			row4b = bdc.Bin2Hex(string.sub(row4,4,-1).."0000")

			-- row 5: (4)00001111, (5)11100000
			local row5 = _data.Rows[5]
			local row5a, row5b = ""
			row5a = bdc.Bin2Hex("0000"..string.sub(row5,1,4))
			row5b = bdc.Bin2Hex(string.sub(row5,5,-1).."00000")

			-- row 6: (5)00011111, (6)11000000
			local row6 = _data.Rows[6]
			local row6a, row6b = ""
			row6a = bdc.Bin2Hex("000"..string.sub(row6,1,5))
			row6b = bdc.Bin2Hex(string.sub(row6,6,-1).."000000")

			-- row 7: (6)00111111, (7)10000000
			local row7 = _data.Rows[7]
			local row7a, row7b = ""
			row7a = bdc.Bin2Hex("00"..string.sub(row7,1,6))
			row7b = bdc.Bin2Hex(string.sub(row7,7,-1).."0000000")
			-----------------------------------------------------
			-- byte 1 (1,2a)
			local byte1 = bdc.BMOr(row1,row2a)
			xformPuzzle[1] = byte1

			-- byte 2 (2b,3a)
			local byte2a = bdc.BMOr(row2b,row3a)
			xformPuzzle[2] = byte2a

			-- byte 3 (3b,4a)
			local byte3 = bdc.BMOr(row3b,row4a)
			xformPuzzle[3] = byte3

			-- byte 4 (4b,5a)
			local byte4 = bdc.BMOr(row4b,row5a)
			xformPuzzle[4] = byte4

			-- byte 5 (5b,6a)
			local byte5 = bdc.BMOr(row5b,row6a)
			xformPuzzle[5] = byte5

			-- byte 6 (6b,7a)
			local byte6 = bdc.BMOr(row6b,row7a)
			xformPuzzle[6] = byte6

			-- byte 7 (7b)
			xformPuzzle[7] = row7b
		----------------------------------------------------------------------------
		-- everything after this point is going to be a bit trickier...
		-- starting at 9x9, each row is composed of at least two bytes.

		-- 9x9
		elseif _data.Size == 9 then
			-- row 1: ( 1)11111111, ( 2)10000000
			local row1 = _data.Rows[1]
			local row1a, row1b = ""
			row1a = bdc.Bin2Hex(string.sub(row1,1,8))
			row1b = bdc.Bin2Hex(string.sub(row1,9,-1).."0000000")

			-- row 2: ( 2)01111111, ( 3)11000000
			local row2 = _data.Rows[2]
			local row2a, row2b = ""
			row2a = bdc.Bin2Hex("0"..string.sub(row2,1,7))
			row2b = bdc.Bin2Hex(string.sub(row2,8,-1).."000000")

			-- row 3: ( 3)00111111, ( 4)11100000
			local row3 = _data.Rows[3]
			local row3a, row3b = ""
			row3a = bdc.Bin2Hex("00"..string.sub(row3,1,6))
			row3b = bdc.Bin2Hex(string.sub(row3,7,-1).."00000")

			-- row 4: ( 4)00011111, ( 5)11110000
			local row4 = _data.Rows[4]
			local row4a, row4b = ""
			row4a = bdc.Bin2Hex("000"..string.sub(row4,1,5))
			row4b = bdc.Bin2Hex(string.sub(row4,6,-1).."0000")

			-- row 5: ( 5)00001111, ( 6)11111000
			local row5 = _data.Rows[5]
			local row5a, row5b = ""
			row5a = bdc.Bin2Hex("0000"..string.sub(row5,1,4))
			row5b = bdc.Bin2Hex(string.sub(row5,5,-1).."000")

			-- row 6: ( 6)00000111, ( 7)11111100
			local row6 = _data.Rows[6]
			local row6a, row6b = ""
			row6a = bdc.Bin2Hex("00000"..string.sub(row6,1,3))
			row6b = bdc.Bin2Hex(string.sub(row6,4,-1).."00")

			-- row 7: ( 7)00000011, ( 8)11111110
			local row7 = _data.Rows[7]
			local row7a, row7b = ""
			row7a = bdc.Bin2Hex("000000"..string.sub(row7,1,2))
			row7b = bdc.Bin2Hex(string.sub(row7,3,-1).."0")

			-- row 8: ( 8)00000001, ( 9)11111111
			local row8 = _data.Rows[8]
			local row8a, row8b = ""
			row8a = bdc.Bin2Hex("0000000"..string.sub(row8,1,1))
			row8b = bdc.Bin2Hex(string.sub(row8,2,-1))

			-- row 9: (10)11111111, (11)10000000
			local row9 = _data.Rows[9]
			local row9a, row9b = ""
			row9a = bdc.Bin2Hex(string.sub(row9,1,8))
			row9b = bdc.Bin2Hex(string.sub(row9,9,-1).."0000000")
			-----------------------------------------------------
			-- byte  1 (1a)
			xformPuzzle[1] = row1a

			-- byte  2 (1b,2a)
			local byte2 = bdc.BMOr(row1b,row2a)
			xformPuzzle[2] = byte2

			-- byte  3 (2b,3a)
			local byte3 = bdc.BMOr(row2b,row3a)
			xformPuzzle[3] = byte3

			-- byte  4 (3b,4a)
			local byte4 = bdc.BMOr(row3b,row4a)
			xformPuzzle[4] = byte4

			-- byte  5 (4b,5a)
			local byte5 = bdc.BMOr(row4b,row5a)
			xformPuzzle[5] = byte5

			-- byte  6 (5b,6a)
			local byte6 = bdc.BMOr(row5b,row6a)
			xformPuzzle[6] = byte6

			-- byte  7 (6b,7a)
			local byte7 = bdc.BMOr(row6b,row7a)
			xformPuzzle[7] = byte7

			-- byte  8 (7b,8a)
			local byte8 = bdc.BMOr(row7b,row8a)
			xformPuzzle[8] = byte8

			-- byte  9 (8b)
			xformPuzzle[9] = row8b

			-- byte 10 (9a)
			xformPuzzle[10] = row9a

			-- byte 11 (9b)
			xformPuzzle[11] = row9b
		----------------------------------------------------------------------------
		-- 10x10
		elseif _data.Size==10 then
			-- row  1: ( 1)11111111 ( 2)11000000
			local row1 = _data.Rows[1]
			local row1a,row1b=""
			row1a = bdc.Bin2Hex(string.sub(row1,1,8))
			row1b = bdc.Bin2Hex(string.sub(row1,9,-1).."000000")

			-- row  2: ( 2)00111111 ( 3)11110000
			local row2 = _data.Rows[2]
			local row2a,row2b=""
			row2a = bdc.Bin2Hex("00"..string.sub(row2,1,6))
			row2b = bdc.Bin2Hex(string.sub(row2,7,-1).."0000")

			-- row  3: ( 3)00001111 ( 4)11111100
			local row3 = _data.Rows[3]
			local row3a,row3b=""
			row3a = bdc.Bin2Hex("0000"..string.sub(row3,1,4))
			row3b = bdc.Bin2Hex(string.sub(row3,5,-1).."00")

			-- row  4: ( 4)00000011 ( 5)11111111
			local row4 = _data.Rows[4]
			local row4a,row4b=""
			row4a = bdc.Bin2Hex("000000"..string.sub(row4,1,2))
			row4b = bdc.Bin2Hex(string.sub(row4,3,-1))

			-- row  5: ( 6)11111111 ( 7)11000000
			local row5 = _data.Rows[5]
			local row5a,row5b=""
			row5a = bdc.Bin2Hex(string.sub(row5,1,8))
			row5b = bdc.Bin2Hex(string.sub(row5,9,-1).."000000")

			-- row  6: ( 7)00111111 ( 8)11110000
			local row6 = _data.Rows[6]
			local row6a,row6b=""
			row6a = bdc.Bin2Hex("00"..string.sub(row6,1,6))
			row6b = bdc.Bin2Hex(string.sub(row6,7,-1).."0000")

			-- row  7: ( 8)00001111 ( 9)11111100
			local row7 = _data.Rows[7]
			local row7a,row7b=""
			row7a = bdc.Bin2Hex("0000"..string.sub(row7,1,4))
			row7b = bdc.Bin2Hex(string.sub(row7,5,-1).."00")

			-- row  8: ( 9)00000011 (10)11111111
			local row8 = _data.Rows[8]
			local row8a,row8b=""
			row8a = bdc.Bin2Hex("000000"..string.sub(row8,1,2))
			row8b = bdc.Bin2Hex(string.sub(row8,3,-1))

			-- row  9: (11)11111111 (12)11000000
			local row9 = _data.Rows[9]
			local row9a,row9b=""
			row9a = bdc.Bin2Hex(string.sub(row9,1,8))
			row9b = bdc.Bin2Hex(string.sub(row9,9,-1).."000000")

			-- row 10: (12)00111111 (13)11110000
			local row10 = _data.Rows[10]
			local row10a,row10b=""
			row10a = bdc.Bin2Hex("00"..string.sub(row10,1,6))
			row10b = bdc.Bin2Hex(string.sub(row10,7,-1).."0000")
			-----------------------------------------------------
			-- byte  1 (1a)
			xformPuzzle[1] = row1a

			-- byte  2 (1b),(2a)
			local byte2 = bdc.BMOr(row1b,row2a)
			xformPuzzle[2] = byte2

			-- byte  3 (2b),(3a)
			local byte3 = bdc.BMOr(row2b,row3a)
			xformPuzzle[3] = byte3

			-- byte  4 (3b),(4a)
			local byte4 = bdc.BMOr(row3b,row4a)
			xformPuzzle[4] = byte3

			-- byte  5 (4b)
			xformPuzzle[5] = row4b

			-- byte  6 (5a)
			xformPuzzle[6] = row5a

			-- byte  7 (5b),(6a)
			local byte7 = bdc.BMOr(row5b,row6a)
			xformPuzzle[7] = byte7

			-- byte  8 (6b),(7a)
			local byte8 = bdc.BMOr(row6b,row7a)
			xformPuzzle[8] = byte8

			-- byte  9 (7b),(8a)
			local byte9 = bdc.BMOr(row7b,row8a)
			xformPuzzle[9] = byte9

			-- byte 10 (8b)
			xformPuzzle[10] = row8b

			-- byte 11 (9a)
			xformPuzzle[11] = row9a

			-- byte 12 (9b),(10a)
			local byte12 = bdc.BMOr(row9b,row10a)
			xformPuzzle[12] = byte9

			-- byte 13 (10b)
			xformPuzzle[13] = row10b

		----------------------------------------------------------------------------
		-- 11x11
		elseif _data.Size==11 then
			-- row  1: ( 1)11111111 ( 2)11100000
			local row1 = _data.Rows[1]
			local row1a,row1b=""
			row1a = bdc.Bin2Hex(string.sub(row1,1,8))
			row1b = bdc.Bin2Hex(string.sub(row1,9,-1).."00000")

			-- row  2: ( 2)00011111 ( 3)11111100
			local row2 = _data.Rows[2]
			local row2a,row2b=""
			row2a = bdc.Bin2Hex("000"..string.sub(row2,1,5))
			row2b = bdc.Bin2Hex(string.sub(row2,6,-1).."00")

			-- row  3: ( 3)00000011 ( 4)11111111 ( 5)10000000
			local row3 = _data.Rows[3]
			local row3a,row3b,row3c=""
			row3a = bdc.Bin2Hex("000000"..string.sub(row3,1,2))
			row3b = bdc.Bin2Hex(string.sub(row3,3,10))
			row3c = bdc.Bin2Hex(string.sub(row3,11,-1).."0000000")

			-- row  4: ( 5)01111111 ( 6)11110000
			local row4 = _data.Rows[4]
			local row4a,row4b=""
			row4a = bdc.Bin2Hex("0"..string.sub(row4,1,7))
			row4b = bdc.Bin2Hex(string.sub(row4,8,-1).."0000")

			-- row  5: ( 6)00001111 ( 7)11111110
			local row5 = _data.Rows[5]
			local row5a,row5b=""
			row5a = bdc.Bin2Hex("0000"..string.sub(row5,1,4))
			row5b = bdc.Bin2Hex(string.sub(row5,5,-1).."0")

			-- row  6: ( 7)00000001 ( 8)11111111 ( 9)11000000
			local row6 = _data.Rows[6]
			local row6a,row6b,row6c=""
			row6a = bdc.Bin2Hex("000000"..string.sub(row6,1,1))
			row6b = bdc.Bin2Hex(string.sub(row6,2,9))
			row6c = bdc.Bin2Hex(string.sub(row6,10,-1).."000000")

			-- row  7: ( 9)00111111 (10)11111000
			local row7 = _data.Rows[7]
			local row7a,row7b=""
			row7a = bdc.Bin2Hex("00"..string.sub(row7,1,6))
			row7b = bdc.Bin2Hex(string.sub(row7,7,-1).."000")

			-- row  8: (10)00000111 (11)11111111
			local row8 = _data.Rows[8]
			local row8a,row8b=""
			row8a = bdc.Bin2Hex("00000"..string.sub(row8,1,3))
			row8b = bdc.Bin2Hex(string.sub(row8,4,-1))

			-- row  9: (12)11111111 (13)11100000
			local row9 = _data.Rows[9]
			local row9a,row9b=""
			row9a = bdc.Bin2Hex(string.sub(row9,1,8))
			row9b = bdc.Bin2Hex(string.sub(row9,9,-1).."00000")

			-- row 10: (13)00011111 (14)11111100
			local row10 = _data.Rows[10]
			local row10a,row10b=""
			row10a = bdc.Bin2Hex("000"..string.sub(row10,1,5))
			row10b = bdc.Bin2Hex(string.sub(row10,6,-1).."00")

			-- row 11: (14)00000011 (15)11111111 (16)10000000
			local row11 = _data.Rows[11]
			local row11a,row11b,row11c=""
			row11a = bdc.Bin2Hex("000000"..string.sub(row11,1,2))
			row11b = bdc.Bin2Hex(string.sub(row11,3,10))
			row11c = bdc.Bin2Hex(string.sub(row11,11,-1).."0000000")
			-----------------------------------------------------
			-- byte  1 (1a)
			xformPuzzle[1] = row1a

			-- byte  2 (1b),(2a)
			local byte2 = bdc.BMOr(row1b,row2a)
			xformPuzzle[2] = byte2

			-- byte  3 (2b),(3a)
			local byte3 = bdc.BMOr(row2b,row3a)
			xformPuzzle[3] = byte3

			-- byte  4 (3b)
			xformPuzzle[4] = row3b

			-- byte  5 (3c),(4a)
			local byte5 = bdc.BMOr(row3c,row4a)
			xformPuzzle[5] = byte5

			-- byte  6 (4b),(5a)
			local byte6 = bdc.BMOr(row4b,row5a)
			xformPuzzle[6] = byte6

			-- byte  7 (5b),(6a)
			local byte7 = bdc.BMOr(row5b,row6a)
			xformPuzzle[7] = byte7

			-- byte  8 (6b)
			xformPuzzle[8] = row6b

			-- byte  9 (6c),(7a)
			local byte9 = bdc.BMOr(row6c,row7a)
			xformPuzzle[9] = byte9

			-- byte 10 (7b),(8a)
			local byte10 = bdc.BMOr(row7b,row8a)
			xformPuzzle[10] = byte10

			-- byte 11 (8b)
			xformPuzzle[11] = row8b

			-- byte 12 (9a)
			xformPuzzle[12] = row9a

			-- byte 13 (9b),(10a)
			local byte13 = bdc.BMOr(row9b,row10a)
			xformPuzzle[13] = byte13

			-- byte 14 (10b),(11a)
			local byte14 = bdc.BMOr(row10b,row11a)
			xformPuzzle[14] = byte14

			-- byte 15 (11b)
			xformPuzzle[15] = row11b

			-- byte 16 (11c)
			xformPuzzle[16] = row11c
		----------------------------------------------------------------------------
		-- 13x13
		elseif _data.Size==13 then
			-- row  1: ( 1)11111111 ( 2)11111000
			local row1 = _data.Rows[1]
			local row1a,row1b=""
			row1a = bdc.Bin2Hex(string.sub(row1,1,8))
			row1b = bdc.Bin2Hex(string.sub(row1,9,-1).."000")

			-- row  2: ( 2)00000111 ( 3)11111111 ( 4)11000000
			local row2 = _data.Rows[2]
			local row2a,row2b,row2c=""
			row2a = bdc.Bin2Hex("00000"..string.sub(row2,1,3))
			row2b = bdc.Bin2Hex(string.sub(row2,4,11))
			row2c = bdc.Bin2Hex(string.sub(row2,12,-1).."000000")

			-- row  3: ( 4)00111111 ( 5)11111110
			local row3 = _data.Rows[3]
			local row3a,row3b=""
			row3a = bdc.Bin2Hex("00"..string.sub(row3,1,6))
			row3b = bdc.Bin2Hex(string.sub(row3,7,-1).."0")

			-- row  4: ( 5)00000001 ( 6)11111111 ( 7)11110000
			local row4 = _data.Rows[4]
			local row4a,row4b,row4c=""
			row4a = bdc.Bin2Hex("0000000"..string.sub(row4,1,1))
			row4b = bdc.Bin2Hex(string.sub(row4,2,9))
			row4c = bdc.Bin2Hex(string.sub(row4,10,-1).."0000")

			-- row  5: ( 7)00001111 ( 8)11111111 ( 9)10000000
			local row5 = _data.Rows[5]
			local row5a,row5b,row5c=""
			row5a = bdc.Bin2Hex("0000"..string.sub(row5,1,4))
			row5b = bdc.Bin2Hex(string.sub(row5,5,12))
			row5c = bdc.Bin2Hex(string.sub(row5,13,-1).."0000000")

			-- row  6: ( 9)01111111 (10)11111100
			local row6 = _data.Rows[6]
			local row6a,row6b=""
			row6a = bdc.Bin2Hex("0"..string.sub(row6,1,7))
			row6b = bdc.Bin2Hex(string.sub(row6,8,-1).."00")

			-- row  7: (10)00000011 (11)11111111 (12)11100000
			local row7 = _data.Rows[7]
			local row7a,row7b,row7c=""
			row7a = bdc.Bin2Hex("000000"..string.sub(row7,1,2))
			row7b = bdc.Bin2Hex(string.sub(row7,3,10))
			row7c = bdc.Bin2Hex(string.sub(row7,11,-1).."00000")

			-- row  8: (12)00011111 (13)11111111
			local row8 = _data.Rows[8]
			local row8a,row8b=""
			row8a = bdc.Bin2Hex("000"..string.sub(row8,1,5))
			row8b = bdc.Bin2Hex(string.sub(row8,6,-1))

			-- row  9: (14)11111111 (15)11111000
			local row9 = _data.Rows[9]
			local row9a,row9b=""
			row9a = bdc.Bin2Hex(string.sub(row9,1,8))
			row9b = bdc.Bin2Hex(string.sub(row9,9,-1).."000")

			-- row 10: (15)00000111 (16)11111111 (17)11000000
			local row10 = _data.Rows[10]
			local row10a,row10b,row10c=""
			row10a = bdc.Bin2Hex("00000"..string.sub(row10,1,3))
			row10b = bdc.Bin2Hex(string.sub(row10,4,11))
			row10c = bdc.Bin2Hex(string.sub(row10,12,-1).."000000")

			-- row 11: (17)00111111 (18)11111110
			local row11 = _data.Rows[11]
			local row11a,row11b=""
			row11a = bdc.Bin2Hex("00"..string.sub(row11,1,6))
			row11b = bdc.Bin2Hex(string.sub(row11,7,-1).."0")

			-- row 12: (18)00000001 (19)11111111 (20)11110000
			local row12 = _data.Rows[12]
			local row12a,row12b,row12c=""
			row12a = bdc.Bin2Hex("0000000"..string.sub(row12,1,1))
			row12b = bdc.Bin2Hex(string.sub(row12,2,9))
			row12c = bdc.Bin2Hex(string.sub(row12,10,-1).."0000")

			-- row 13: (20)00001111 (21)11111111 (22)10000000
			local row13 = _data.Rows[13]
			local row13a,row13b,row13c=""
			row13a = bdc.Bin2Hex("0000"..string.sub(row13,1,4))
			row13b = bdc.Bin2Hex(string.sub(row13,5,12))
			row13c = bdc.Bin2Hex(string.sub(row13,13,-1).."0000000")
			-----------------------------------------------------
			-- byte  1 (1a)
			xformPuzzle[1] = row1a

			-- byte  2 (1b),(2a)
			local byte2 = bdc.BMOr(row1b,row2a)
			xformPuzzle[2] = byte2

			-- byte  3 (2b)
			xformPuzzle[3] = row2b

			-- byte  4 (2c),(3a)
			local byte4 = bdc.BMOr(row2c,row3a)
			xformPuzzle[4] = byte4

			-- byte  5 (3b),(4a)
			local byte5 = bdc.BMOr(row3b,row4a)
			xformPuzzle[5] = byte5

			-- byte  6 (4b)
			xformPuzzle[6] = row4b

			-- byte  7 (4c),(5a)
			local byte7 = bdc.BMOr(row4c,row5a)
			xformPuzzle[7] = byte7

			-- byte  8 (5b)
			xformPuzzle[8] = row5b

			-- byte  9 (5c),(6a)
			local byte9 = bdc.BMOr(row5c,row6a)
			xformPuzzle[9] = byte9

			-- byte 10 (6b),(7a)
			local byte10 = bdc.BMOr(row6b,row7a)
			xformPuzzle[10] = byte10

			-- byte 11 (7b)
			xformPuzzle[11] = row7b

			-- byte 12 (7c),(8a)
			local byte12 = bdc.BMOr(row7c,row8a)
			xformPuzzle[12] = byte12

			-- byte 13 (8b)
			xformPuzzle[13] = row8b

			-- byte 14 (9a)
			xformPuzzle[14] = row9a

			-- byte 15 (9b),(10a)
			local byte15 = bdc.BMOr(row9b,row10a)
			xformPuzzle[15] = byte15

			-- byte 16 (10b)
			xformPuzzle[16] = row10b

			-- byte 17 (10c),(11a)
			local byte17 = bdc.BMOr(row10c,row11a)
			xformPuzzle[17] = byte17

			-- byte 18 (11b),(12a)
			local byte18 = bdc.BMOr(row11b,row12a)
			xformPuzzle[17] = byte18

			-- byte 19 (12b)
			xformPuzzle[19] = row12b

			-- byte 20 (12c),(13a)
			local byte20 = bdc.BMOr(row12c,row13a)
			xformPuzzle[20] = byte20

			-- byte 21 (13b)
			xformPuzzle[21] = row13b

			-- byte 22 (13c)
			xformPuzzle[22] = row13c
		----------------------------------------------------------------------------
		-- 14x14
		elseif _data.Size==14 then
			-- row  1: ( 1)11111111 ( 2)11111100
			local row1 = _data.Rows[1]
			local row1a,row1b=""
			row1a = bdc.Bin2Hex(string.sub(row1,1,8))
			row1b = bdc.Bin2Hex(string.sub(row1,9,-1).."00")

			-- row  2: ( 2)00000011 ( 3)11111111 ( 4)11110000
			local row2 = _data.Rows[2]
			local row2a,row2b,row2c=""
			row2a = bdc.Bin2Hex("000000"..string.sub(row2,1,2))
			row2b = bdc.Bin2Hex(string.sub(row2,3,10))
			row2c = bdc.Bin2Hex(string.sub(row2,11,-1).."0000")

			-- row  3: ( 4)00001111 ( 5)11111111 ( 6)11000000
			local row3 = _data.Rows[3]
			local row3a,row3b,row3c=""
			row3a = bdc.Bin2Hex("0000"..string.sub(row3,1,4))
			row3b = bdc.Bin2Hex(string.sub(row3,5,12))
			row3c = bdc.Bin2Hex(string.sub(row3,13,-1).."000000")

			-- row  4: ( 6)00111111 ( 7)11111111
			local row4 = _data.Rows[4]
			local row4a,row4b=""
			row4a = bdc.Bin2Hex("00"..string.sub(row4,1,6))
			row4b = bdc.Bin2Hex(string.sub(row4,7,-1))

			-- row  5: ( 8)11111111 ( 9)11111100
			local row5 = _data.Rows[5]
			local row5a,row5b=""
			row5a = bdc.Bin2Hex(string.sub(row5,1,8))
			row5b = bdc.Bin2Hex(string.sub(row5,9,-1).."00")

			-- row  6: ( 9)00000011 (10)11111111 (11)11110000
			local row6 = _data.Rows[6]
			local row6a,row6b,row6c=""
			row6a = bdc.Bin2Hex("000000"..string.sub(row6,1,2))
			row6b = bdc.Bin2Hex(string.sub(row6,3,10))
			row6c = bdc.Bin2Hex(string.sub(row6,11,-1).."0000")

			-- row  7: (11)00001111 (12)11111111 (13)11000000
			local row7 = _data.Rows[7]
			local row7a,row7b,row7c=""
			row7a = bdc.Bin2Hex("0000"..string.sub(row7,1,4))
			row7b = bdc.Bin2Hex(string.sub(row7,5,12))
			row7c = bdc.Bin2Hex(string.sub(row7,13,-1).."000000")

			-- row  8: (13)00111111 (14)11111111
			local row8 = _data.Rows[8]
			local row8a,row8b=""
			row8a = bdc.Bin2Hex("00"..string.sub(row8,1,6))
			row8b = bdc.Bin2Hex(string.sub(row8,7,-1))

			-- row  9: (15)11111111 (16)11111100
			local row9 = _data.Rows[9]
			local row9a,row9b=""
			row9a = bdc.Bin2Hex(string.sub(row9,1,8))
			row9b = bdc.Bin2Hex(string.sub(row9,9,-1).."00")

			-- row 10: (16)00000011 (17)11111111 (18)11110000
			local row10 = _data.Rows[10]
			local row10a,row10b,row10c=""
			row10a = bdc.Bin2Hex("000000"..string.sub(row10,1,2))
			row10b = bdc.Bin2Hex(string.sub(row10,3,10))
			row10c = bdc.Bin2Hex(string.sub(row10,11,-1).."0000")

			-- row 11: (18)00001111 (19)11111111 (20)11000000
			local row11 = _data.Rows[11]
			local row11a,row11b,row11c=""
			row11a = bdc.Bin2Hex("0000"..string.sub(row11,1,4))
			row11b = bdc.Bin2Hex(string.sub(row11,5,12))
			row11c = bdc.Bin2Hex(string.sub(row11,13,-1).."000000")

			-- row 12: (20)00111111 (21)11111111
			local row12 = _data.Rows[12]
			local row12a,row12b=""
			row12a = bdc.Bin2Hex("00"..string.sub(row12,1,6))
			row12b = bdc.Bin2Hex(string.sub(row12,7,-1))

			-- row 13: (22)11111111 (23)11111100
			local row13 = _data.Rows[13]
			local row13a,row13b=""
			row13a = bdc.Bin2Hex(string.sub(row13,1,8))
			row13b = bdc.Bin2Hex(string.sub(row13,9,-1).."00")

			-- row 14: (23)00000011 (24)11111111 (25)11110000
			local row14 = _data.Rows[14]
			local row14a,row14b,row14c=""
			row14a = bdc.Bin2Hex("000000"..string.sub(row14,1,2))
			row14b = bdc.Bin2Hex(string.sub(row14,3,10))
			row14c = bdc.Bin2Hex(string.sub(row14,11,-1).."0000")
			-----------------------------------------------------
			-- byte  1 (1a)
			xformPuzzle[1] = row1a

			-- byte  2 (1b),(2a)
			local byte2 = bdc.BMOr(row1b,row2a)
			xformPuzzle[2] = byte2

			-- byte  3 (2b)
			xformPuzzle[3] = row2b

			-- byte  4 (2c),(3a)
			local byte4 = bdc.BMOr(row2c,row3a)
			xformPuzzle[4] = byte4

			-- byte  5 (3b)
			xformPuzzle[5] = row3b

			-- byte  6 (3c),(4a)
			local byte6 = bdc.BMOr(row3c,row4a)
			xformPuzzle[6] = byte6

			-- byte  7 (4b)
			xformPuzzle[7] = row4b

			-- byte  8 (5a)
			xformPuzzle[8] = row5a

			-- byte  9 (5b),(6a)
			local byte9 = bdc.BMOr(row5b,row6a)
			xformPuzzle[9] = byte9

			-- byte 10 (6b)
			xformPuzzle[10] = row6b

			-- byte 11 (6c),(7a)
			local byte11 = bdc.BMOr(row6c,row7a)
			xformPuzzle[11] = byte11

			-- byte 12 (7b)
			xformPuzzle[12] = row7b

			-- byte 13 (7c),(8a)
			local byte13 = bdc.BMOr(row7c,row8a)
			xformPuzzle[13] = byte13

			-- byte 14 (8b)
			xformPuzzle[14] = row8b

			-- byte 15 (9a)
			xformPuzzle[15] = row9a

			-- byte 16 (9b),(10a)
			local byte16 = bdc.BMOr(row9b,row10a)
			xformPuzzle[16] = byte16

			-- byte 17 (10b)
			xformPuzzle[17] = row10b

			-- byte 18 (10c),(11a)
			local byte18 = bdc.BMOr(row10c,row11a)
			xformPuzzle[18] = byte18

			-- byte 19 (11b)
			xformPuzzle[19] = row11b

			-- byte 20 (11c),(12a)
			local byte20 = bdc.BMOr(row11c,row12a)
			xformPuzzle[20] = byte20

			-- byte 21 (12b)
			xformPuzzle[21] = row12b

			-- byte 22 (13a)
			xformPuzzle[22] = row13a

			-- byte 23 (13b),(14a)
			local byte23 = bdc.BMOr(row13b,row14a)
			xformPuzzle[23] = byte23

			-- byte 24 (14b)
			xformPuzzle[24] = row14b

			-- byte 25 (14c)
			xformPuzzle[25] = row14c
		----------------------------------------------------------------------------
		-- 15x15
		elseif _data.Size==15 then
			-- row  1: ( 1)11111111 ( 2)11111110
			local row1 = _data.Rows[1]
			local row1a,row1b=""
			row1a = bdc.Bin2Hex(string.sub(row1,1,8))
			row1b = bdc.Bin2Hex(string.sub(row1,9,-1).."0")

			-- row  2: ( 2)00000001 ( 3)11111111 ( 4)11111100
			local row2 = _data.Rows[2]
			local row2a,row2b,row2c=""
			row2a = bdc.Bin2Hex("0000000"..string.sub(row2,1,1))
			row2b = bdc.Bin2Hex(string.sub(row2,2,9))
			row2c = bdc.Bin2Hex(string.sub(row2,10,-1).."00")

			-- row  3: ( 4)00000011 ( 5)11111111 ( 6)11111000
			local row3 = _data.Rows[3]
			local row3a,row3b,row3c=""
			row3a = bdc.Bin2Hex("000000"..string.sub(row3,1,2))
			row3b = bdc.Bin2Hex(string.sub(row3,3,10))
			row3c = bdc.Bin2Hex(string.sub(row3,11,-1).."000")

			-- row  4: ( 6)00000111 ( 7)11111111 ( 8)11110000
			local row4 = _data.Rows[4]
			local row4a,row4b,row4c=""
			row4a = bdc.Bin2Hex("00000"..string.sub(row4,1,3))
			row4b = bdc.Bin2Hex(string.sub(row4,4,11))
			row4c = bdc.Bin2Hex(string.sub(row4,12,-1).."0000")

			-- row  5: ( 8)00001111 ( 9)11111111 (10)11100000
			local row5 = _data.Rows[5]
			local row5a,row5b,row5c=""
			row5a = bdc.Bin2Hex("0000"..string.sub(row5,1,4))
			row5b = bdc.Bin2Hex(string.sub(row5,5,12))
			row5c = bdc.Bin2Hex(string.sub(row5,13,-1).."00000")

			-- row  6: (10)00011111 (11)11111111 (12)11000000
			local row6 = _data.Rows[6]
			local row6a,row6b,row6c=""
			row6a = bdc.Bin2Hex("000"..string.sub(row6,1,5))
			row6b = bdc.Bin2Hex(string.sub(row6,6,13))
			row6c = bdc.Bin2Hex(string.sub(row6,14,-1).."000000")

			-- row  7: (12)00111111 (13)11111111 (14)10000000
			local row7 = _data.Rows[7]
			local row7a,row7b,row7c=""
			row7a = bdc.Bin2Hex("00"..string.sub(row7,1,6))
			row7b = bdc.Bin2Hex(string.sub(row7,7,14))
			row7c = bdc.Bin2Hex(string.sub(row7,15,-1).."0000000")

			-- row  8: (14)01111111 (15)11111111
			local row8 = _data.Rows[8]
			local row8a,row8b=""
			row8a = bdc.Bin2Hex("0"..string.sub(row8,1,7))
			row8b = bdc.Bin2Hex(string.sub(row8,8,-1))

			-- row  9: (16)11111111 (17)11111110
			local row9 = _data.Rows[9]
			local row9a,row9b=""
			row9a = bdc.Bin2Hex(string.sub(row9,1,8))
			row9b = bdc.Bin2Hex(string.sub(row9,9,-1).."0")

			-- row 10: (17)00000001 (18)11111111 (19)11111100
			local row10 = _data.Rows[10]
			local row10a,row10b,row10c=""
			row10a = bdc.Bin2Hex("0000000"..string.sub(row10,1,1))
			row10b = bdc.Bin2Hex(string.sub(row10,2,9))
			row10c = bdc.Bin2Hex(string.sub(row10,10,-1).."00")

			-- row 11: (19)00000011 (20)11111111 (21)11111000
			local row11 = _data.Rows[11]
			local row11a,row11b,row11c=""
			row11a = bdc.Bin2Hex("000000"..string.sub(row11,1,2))
			row11b = bdc.Bin2Hex(string.sub(row11,3,10))
			row11c = bdc.Bin2Hex(string.sub(row11,11,-1).."000")

			-- row 12: (21)00000111 (22)11111111 (23)11110000
			local row12 = _data.Rows[12]
			local row12a,row12b,row12c=""
			row12a = bdc.Bin2Hex("00000"..string.sub(row12,1,3))
			row12b = bdc.Bin2Hex(string.sub(row12,4,11))
			row12c = bdc.Bin2Hex(string.sub(row12,12,-1).."0000")

			-- row 13: (23)00001111 (24)11111111 (25)11100000
			local row13 = _data.Rows[13]
			local row13a,row13b,row13c=""
			row13a = bdc.Bin2Hex("0000"..string.sub(row13,1,4))
			row13b = bdc.Bin2Hex(string.sub(row13,5,12))
			row13c = bdc.Bin2Hex(string.sub(row13,13,-1).."00000")

			-- row 14: (25)00011111 (26)11111111 (27)11000000
			local row14 = _data.Rows[14]
			local row14a,row14b,row14c=""
			row14a = bdc.Bin2Hex("000"..string.sub(row14,1,5))
			row14b = bdc.Bin2Hex(string.sub(row14,6,13))
			row14c = bdc.Bin2Hex(string.sub(row14,14,-1).."000000")

			-- row 15: (27)00111111 (28)11111111 (29)10000000
			local row15 = _data.Rows[15]
			local row15a,row15b,row15c=""
			row15a = bdc.Bin2Hex("00"..string.sub(row15,1,6))
			row15b = bdc.Bin2Hex(string.sub(row15,7,14))
			row15c = bdc.Bin2Hex(string.sub(row15,15,-1).."0000000")
			-----------------------------------------------------
			-- byte  1 (1a)
			xformPuzzle[1] = row1a

			-- byte  2 (1b),(2a)
			local byte2 = bdc.BMOr(row1b,row2a)
			xformPuzzle[2] = byte2

			-- byte  3 (2b)
			xformPuzzle[3] = row2b

			-- byte  4 (2c),(3a)
			local byte4 = bdc.BMOr(row2c,row3a)
			xformPuzzle[4] = byte4

			-- byte  5 (3b)
			xformPuzzle[5] = row3b

			-- byte  6 (3c),(4a)
			local byte6 = bdc.BMOr(row3c,row4a)
			xformPuzzle[6] = byte6

			-- byte  7 (4b)
			xformPuzzle[7] = row4b

			-- byte  8 (4c),(5a)
			local byte8 = bdc.BMOr(row4c,row5a)
			xformPuzzle[8] = byte8

			-- byte  9 (5b)
			xformPuzzle[9] = row5b

			-- byte 10 (5c),(6a)
			local byte10 = bdc.BMOr(row5c,row6a)
			xformPuzzle[10] = byte10

			-- byte 11 (6b)
			xformPuzzle[11] = row6b

			-- byte 12 (6c),(7a)
			local byte12 = bdc.BMOr(row6c,row7a)
			xformPuzzle[12] = byte12

			-- byte 13 (7b)
			xformPuzzle[13] = row7b

			-- byte 14 (7c),(8a)
			local byte14 = bdc.BMOr(row7c,row8a)
			xformPuzzle[14] = byte14

			-- byte 15 (8b)
			xformPuzzle[15] = row8b

			-- byte 16 (9a)
			xformPuzzle[16] = row9a

			-- byte 17 (9b),(10a)
			local byte17 = bdc.BMOr(row9b,row10a)
			xformPuzzle[17] = byte17

			-- byte 18 (10b)
			xformPuzzle[18] = row10b

			-- byte 19 (10c),(11a)
			local byte19 = bdc.BMOr(row10c,row11a)
			xformPuzzle[19] = byte19

			-- byte 20 (11b)
			xformPuzzle[20] = row11b

			-- byte 21 (11c),(12a)
			local byte21 = bdc.BMOr(row11c,row12a)
			xformPuzzle[21] = byte21

			-- byte 22 (12b)
			xformPuzzle[22] = row12b

			-- byte 23 (12c),(13a)
			local byte23 = bdc.BMOr(row12c,row13a)
			xformPuzzle[23] = byte23

			-- byte 24 (13b)
			xformPuzzle[24] = row13b

			-- byte 25 (13c),(14a)
			local byte25 = bdc.BMOr(row13c,row14a)
			xformPuzzle[25] = byte25

			-- byte 26 (14b)
			xformPuzzle[26] = row14b

			-- byte 27 (14c),(15a)
			local byte27 = bdc.BMOr(row14c,row15a)
			xformPuzzle[27] = byte27

			-- byte 28 (15b)
			xformPuzzle[28] = row15b

			-- byte 29 (15c)
			xformPuzzle[29] = row15c
		end
	end

	local difficulty = bdc.Dec2Bin(_data.DifficultyNum).."0000"
	local puzzleSize = _data.Size - 5
	local puzzleSizeBin = "0000"..bdc.Hex2Bin(bdc.Dec2Hex(puzzleSize))

	local combinedSizeDiff = bdc.BMOr(bdc.Bin2Hex(difficulty),bdc.Bin2Hex(puzzleSizeBin))

	if #xformPuzzle > 0 then
		local byteCount = 0

		-- get puzzle data
		local out = ""
		for k,v in pairs(xformPuzzle) do
			if string.len(v) == 4 then
				-- is a word; need to un-word it.
				local b1 = string.sub(v,1,2)
				local b2 = string.sub(v,3,-1)
				out = out..string.format("0x%s,0x%s",b1,b2)
				byteCount = byteCount+2
			else
				-- is a byte
				out = out..string.format("0x%s",v)
				byteCount = byteCount+1
			end
			if k < #xformPuzzle then out = out.."," end
		end

		print(string.format("// %s: [%dx%d; %s] %s",_levelnum,_data.Size,_data.Size,_data.Difficulty,_data.Name))
		print(string.format("const byte puzzle%s[%2d]={ 0x%s,%s }; ",_levelnum,byteCount+1,combinedSizeDiff,out))
		print(string.format("const byte   name%s[%2d]={ \"%s\\xFF\" };",_levelnum,#_data.Name+1,_data.Name))
		print()
	else
		print(string.format("// %s: [%dx%d; %s] %s",_levelnum,_data.Size,_data.Size,_data.Difficulty,_data.Name))
		print(string.format("// Export for %dx%d not done yet",_data.Size,_data.Size))
		print();
	end
end

local dataPointers = {}
local namePointers = {}

-- search for all "###.level" files in directory
for file in lfs.dir("./") do
	if string.find(file,".level") then
		-- get level number from file
		local levelNum = string.sub(file,1,3)

		-- open file
		local levelFile = io.open(file,"r")
		if levelFile == nil then
			print(string.format("Could not open '%s' for reading.",file))
			io.close(levelFile)
			return
		end

		-- read contents
		local thisLevel = ReadLevelFile(levelFile)
		io.close(levelFile)

		-- and work on the contents
		ConvertPuzzle(thisLevel,levelNum)

		dataPointers[#dataPointers+1] = string.format("puzzle%s",levelNum)
		namePointers[#namePointers+1] = string.format("name%s",levelNum)
	end
end

-- and write data pointers at the end
print("// Normal mode puzzle data pointers")
print(string.format("const byte* const puzzleData[%d]={",#dataPointers))

local dp = ""
for i=1,#dataPointers do
	dp = dp..dataPointers[i]
	if i~=#dataPointers then dp=dp.."," end
end
print("\t"..dp)

print("};")
print()

print("// Normal mode puzzle name pointers")
print(string.format("const byte* const puzzleNames[%d]={",#namePointers))

local np = ""
for i=1,#namePointers do
	np = np..namePointers[i]
	if i~=#namePointers then np=np.."," end
end
print("\t"..np)

print("};")