/// debug mode nametables
#include "_nam/nam_dbgmain.h"
#include "_nam/nam_dbgchr.h"
#include "_nam/nam_dbgmem.h"

// bigtile cursor
const byte chrTest_blockX[4]={ 192,208,192,208 };
const byte chrTest_blockY[4]={ 95,95,111,111 };
const byte cursorTile = 0x0A;
const byte chrTest_cAttr2[4]={ 0x22,0x62,0xA2,0xE2 };

// debuggery (zero page var)
static byte dbgChr_bank;

/** Debug Stuff ***************************************************************/
// chrtest sprite palette
const byte chrTest_sprPalBlock1[]={
	0,0,0x05,0, 8,0,0x01,0,
	0,8,0x02,0, 8,8,0x03,0,
	128
};
const byte chrTest_sprPalBlock2[]={
	0,0,0x05,1, 8,0,0x01,1,
	0,8,0x02,1, 8,8,0x03,1,
	128
};
const byte chrTest_sprPalBlock3[]={
	0,0,0x05,2, 8,0,0x01,2,
	0,8,0x02,2, 8,8,0x03,2,
	128
};
const byte chrTest_sprPalBlock4[]={
	0,0,0x05,3, 8,0,0x01,3,
	0,8,0x02,3, 8,8,0x03,3,
	128
};
const byte* const sprList_chrTest[]={
	chrTest_sprPalBlock1, chrTest_sprPalBlock2,
	chrTest_sprPalBlock3, chrTest_sprPalBlock4
};

void debugCharset()
{
	oam_clear();
	bright=4;
	pal_bright(bright);
	unrle_vram(nam_dbgchr,0x2000);
	pal_bg(palBG_Title);
	pal_spr(palSPR_Sprite1);

	// write charset to VRAM manually since it does not RLE nicely.
	for(j=0;j<16;j++){
		vram_adr(0x20C6+(j*0x20));
		for(i=0;i<16;i++){
			vram_put(i+(j*16));
		}
	}

	// reset some vars
	pad=0;
	i=0;
	j=0;
	dbgChr_bank=0;

	// do any spritey things

	ppu_on_all();

	while(1){
		curSprite=9<<4;
		// display sprite palettes
		for(i=0;i<4;++i){
			curSprite=oam_meta_spr(chrTest_blockX[i],chrTest_blockY[i],curSprite,sprList_chrTest[i]);
		}

		// add 16px tile cursor
		for(i=0;i<4;++i){
			// in front
			curSprite=oam_spr(184+spr_cursorX[i],143+spr_cursorY[i],cursorTile,chrTest_cAttr[i],curSprite);
			// behind
			curSprite=oam_spr(200+spr_cursorX[i],143+spr_cursorY[i],cursorTile,chrTest_cAttr2[i],curSprite);
		}
		// add 8px tile cursors
		curSprite=oam_spr(216,143,0x0B,3,curSprite); // front
		curSprite=oam_spr(216,151,0x0B,0x21,curSprite); // behind

		ppu_waitnmi();
		++frameCount;

		// save old value of dbgChr_bank
		j=dbgChr_bank;

		pad=pad_trigger(0);
		if(pad&PAD_START){
			curScreen=0x40;
			subScreen=0;
			break;
		}
		else if(pad&PAD_A){
			dbgChr_bank=0;
			ppu_off();
			bank_bg(0); bank_spr(1);
			ppu_on_all();
		}
		else if(pad&PAD_B){
			dbgChr_bank=1;
			ppu_off();
			bank_bg(1); bank_spr(0);
			ppu_on_all();
		}

		// draw bank name
		j=(dbgChr_bank==0)?0x06:0x21;
		curSprite=oam_spr(32,31,j,0,curSprite);
		ppu_waitnmi();
	}

	// prepare next screen
	bank_bg(0);
	bank_spr(1);
	reset_vram(1);
}

void memView_SwitchMode(void){
	oam_clear();

	// I probably shouldn't be doing it like this but I don't think
	// I care too much for a debugging screen.
	ppu_off();

	// set old
	vram_adr(0x20B6+(m*3));
	vram_put(0x91);

	// change mode
	if(m<2) ++m;
	else m=0;

	// set new
	vram_adr(0x20B6+(m*3));
	vram_put(0x90);

	// set address dot
	vram_adr(0x2076);
	vram_put(m==1?0x90:0x91);

	ppu_on_all();
}

void memView_UpdateValues(void){
	if(m==0){ oam_clear(); }
	ppu_off();

	p=0; t=0;
	// top: +#
	for(i=0;i<8;i++){
		vram_adr(0x20E8+(i*3));
		vram_put(4+i);
	}

	p=0; t=0;
	// numbers down the side (works mostly ok...)
	for(i=0;i<18;i++){
		vram_adr(0x2122+(i*0x20));

		// first (needs to be better at boundaries; see 0x0F80 for example)
		t=(k16&0xF000)>>12;
		if(t>9 && t<16){ s=0x20+(t-10); }
		else if((p+t)>15){ s=4+(t-16); }
		else{ s=t+4; }
		vram_put(s);

		// second (see above comment but more often)
		t=(k16&0x0F00)>>8;
		if(t>9 && t<16){
			s=0x20+(t-10);
		}
		else if(t>15){
			s=4+(t-16);
		}
		else{ s=t+4; }
		vram_put(s);

		// third
		t=(k16&0x00F0)>>4;
		if(p+t>9 && p+t<16){
			s=0x20+((p+t)-10);
		}
		else if((p+t)>15){
			s=4+((p+t)-16);
		}
		else{ s=p+t+4; }
		vram_put(s);

		// 4th is static
		switch(i%2){
			case 0: vram_put(0x04); break;
			case 1: vram_put(0x0C); ++p; break;
			default: break;
		}
	}

	// fill in middle values
	p=0; t=0;
	for(u=0;u<18;u++){
		vram_adr(0x2127+(u*0x20));
		for(v=0;v<8;v++){
			p=*((byte*)(k16+((u*8)+v)));

			t=(p&0xF0)>>4;
			if(t>9){ s=0x20+(t-10); }
			else{ s=t+4; }
			vram_put(s);

			t=(p&0x0F);
			if(t>9){ s=0x20+(t-10); }
			else{ s=t+4; }
			vram_put(s);

			vram_put(0);
		}
	}
	ppu_on_all();
}

void debugMemView(void){
	oam_clear();
	bright=4;
	pal_bright(bright);
	unrle_vram(nam_dbgmem,0x2000);
	pal_bg(palBG_Title);
	pal_spr(palSPR_Sprite1);

	pad=0;
	k16=0; // current address to view

	memView_UpdateValues();

	i=0;
	j=0;	// cur. row
	k=0;	// cur. column
	m=0;	// memview mode; 0=view, 1=edit addr, 2=edit data
	n=0;	// addr. edit pos
	p=0;

	ppu_on_all();

	while(1){
		curSprite=1<<4;
		if(m==1){ // edit address
			curSprite=oam_spr(192+(n*8),31,0x0F,3,curSprite);
		}
		else if(m==2){ // edit values
			p=k+1;
			if(k>0 && p%3==0){ p=8; } else{ p=0; }
			curSprite=oam_spr(56+(k*8)+p,71+(j*8),0x02,OAM_BEHIND|2,curSprite);
		}

		if(m==1){
			ppu_off();
			vram_adr(0x2078);

			t=(k16&0xF000)>>12;
			if(t>9){ s=0x20+(t-10); }
			else{ s=t+4; }
			vram_put(s);

			t=(k16&0x0F00)>>8;
			if(t>9){ s=0x20+(t-10); }
			else{ s=t+4; }
			vram_put(s);

			t=(k16&0x00F0)>>4;
			if(t>9){ s=0x20+(t-10); }
			else{ s=t+4; }
			vram_put(s);

			t=(k16&0x000F);
			if(t>9){ s=0x20+(t-10); }
			else{ s=t+4; }
			vram_put(s);

			ppu_on_all();
			t=0;
		}

		if(u==1){
			u=0;
			memView_UpdateValues();
		}

		ppu_waitnmi();
		frameCount++;

		pad=pad_trigger(0);
		if(pad&PAD_START){ subScreen=0; break; }	// exit
		if(pad&PAD_SELECT){	// change modes
			memView_SwitchMode();
		}

		// based on what mode is active, the buttons do different things.
		if(m==0){
			// view mode
			if(pad&PAD_A){ u=1; }
			continue;
		}
		else if(m==1){
			// edit addr
			// todo: allow direction wrapping?
			if(pad&PAD_LEFT){ if(n>0) --n; }
			if(pad&PAD_RIGHT){ if(n<2) ++n; }
			if(pad&PAD_UP){
				switch(n){
					case 0: p = (k16&0xF000)>>12; break;
					case 1: p = (k16&0x0F00)>>8; break;
					case 2: p = (k16&0x00F0)>>4; break;
					//case 3: p = (k16&0x000F); break;
					default: break;
				}
				// increase value at cursor
				if(p<0xF){ ++p; }
				else{ p = 0; }
				// reconstruct k16
				switch(n){
					case 0: k16 = (k16&0x0FFF)|p<<12; break;
					case 1: k16 = (k16&0xF0FF)|p<<8; break;
					case 2: k16 = (k16&0xFF0F)|p<<4; break;
					//case 3: k16 = (k16&0xFFF0)|p; break;
					default: break;
				}
			}
			if(pad&PAD_DOWN){
				// decrease value at cursor
				switch(n){
					case 0: p = (k16&0xF000)>>12; break;
					case 1: p = (k16&0x0F00)>>8; break;
					case 2: p = (k16&0x00F0)>>4; break;
					//case 3: p = (k16&0x000F); break;
					default: break;
				}
				if(p>0){ --p; }
				else{ p = 0xF; }
				// reconstruct k16
				switch(n){
					case 0: k16 = (k16&0x0FFF)|p<<12; break;
					case 1: k16 = (k16&0xF0FF)|p<<8; break;
					case 2: k16 = (k16&0xFF0F)|p<<4; break;
					//case 3: k16 = (k16&0xFFF0)|p; break;
					default: break;
				}
			}
			if(pad&PAD_A){ u=1; }
		}
		else if(m==2){
			// edit data
			if(pad&PAD_LEFT){ if(k>0){ --k; pad=0; } }
			if(pad&PAD_RIGHT){ if(k<22){ ++k; pad=0; } }
			if(pad&PAD_UP){ if(j>0){ --j; pad=0; } }
			if(pad&PAD_DOWN){ if(j<17){ ++j; pad=0; } }
			// a: inc byte; b: dec byte

			// todo: make repeating input not suck
			//pad=pad_state(0);
			//if(pad&PAD_LEFT && !(frameCount&5)){ if(k>0) --k; }
			//if(pad&PAD_RIGHT && !(frameCount&5)){ if(k<22) ++k; }
			//if(pad&PAD_UP && !(frameCount&5)){ if(j>0) --j; }
			//if(pad&PAD_DOWN && !(frameCount&5)){ if(j<17) ++j; }
		}
	}

	// prepare next screen
	reset_vram(1);
}

void debugMainMenu(void){
	oam_clear();
	bright=4;
	pal_bright(bright);
	unrle_vram(nam_dbgmain,0x2000);
	pal_bg(palBG_Title);
	pal_spr(palSPR_Sprite1);

	pad=0;

	// write build info
	// variant (blargh this is going to be a jerk.)
	vram_adr(0x22E3);
	for(i=0;i<sizeof(verVariant)-1;i++){
		if(verVariant[i] == '-'){
			vram_put(0xC5);
		}
		else if(verVariant[i] >= 0x61 && verVariant[i] <= 0x7A){
			vram_put(verVariant[i]-0x41);
		}
		else{
			vram_put(verVariant[i]-0x21);
		}
	}
	// date
	vram_adr(0x2B03); for(i=0;i<4;i++){ vram_put(verDate[i]-0x2C); } // year
	vram_adr(0x2B08); for(i=0;i<2;i++){ vram_put(verDate[i+5]-0x2C); } // month
	vram_adr(0x2B0B); for(i=0;i<2;i++){ vram_put(verDate[i+8]-0x2C); } // day
	// time
	vram_adr(0x2B23); for(i=0;i<2;i++){ vram_put(verTime[i]-0x2C); } // hour
	vram_adr(0x2B26); for(i=0;i<2;i++){ vram_put(verTime[i+3]-0x2C); } // minute
	// version
	vram_adr(0x2329); vram_put(verNum[1]-0x2C);
	vram_adr(0x232B); vram_put(verNum[3]-0x2C); vram_put(verNum[4]-0x2C);

	i=0;
	j=0;
	k=0;
	m=0;	// debug menu cursor

	ppu_on_all();

	while(1){
		curSprite=1<<2;
		curSprite=oam_spr(22,30+(m*8),0x0C,2,curSprite);

		ppu_waitnmi();
		frameCount++;

		pad=pad_trigger(0);
		if(pad&PAD_A || pad&PAD_START){
			if(m==17){ curScreen=0; break; } // exit
			// check m and set sub-screen or whatever
			if(m==0){ subScreen=1; break; }	// character test
			if(m==1){ subScreen=2; break; }	// memview
		}

		if(pad&PAD_UP){ if(m>0) --m; }
		if(pad&PAD_DOWN){ if(m<17) ++m; }
	}

	// prepare next screen
	reset_vram(1);
}


// main() switch:
/*
case 0x40:
	if(subScreen==0){ debugMainMenu(); }
	else{
		// handle subscreens
		if(subScreen==1){ debugCharset(); }
		else if(subScreen==2){ debugMemView(); }
	}
	break;
*/

/** newer debugging code snippets **/
// code from game_swapScreens(), just before "clear board on restart"
// in debug mode, update the player info
	/*
	if(pauseStatus && *(byte*)GAMEDEBUG_ACTIVE){
		ppu_off();
		// gamePuzzleData dump (0x2740)
		vram_adr(0x2740);
		for(i=0;i<32;i++){
			n=gamePuzzleData[i];
			// lower
			q=(n&0x0F);
			if(q>9){ vram_put(32+q); }
			else{ vram_put(4+q); }
		}
		vram_adr(0x2760);
		for(i=0;i<32;i++){
			n=gamePuzzleData[i+32];
			// lower
			q=(n&0x0F);
			if(q>9){ vram_put(32+q); }
			else{ vram_put(4+q); }
		}
		ppu_on_all();
	}
	*/