/* family picross (NTSC-freem), a game by freemco */
#include "neslib.h"
#include "fr_tools.h"
#include "verInfo.h"

#include "picross.h"
#define _DEBUGMODE_ 0

/* asm imports */
// * reset-related * //
void __fastcall__ addResetSignature(void);
byte __fastcall__ checkResetSignature(void);
// * sram-related * //
unsigned char __fastcall__ sramCheckHeader(void);

#define MAIN_MENU_UPDATE_LIST_COUNT 32
byte updListData_menuMain[]={
	// tile1
	0x22,0xA8,BIGTILE_MENU_DOWN,
	0x22,0xA9,BIGTILE_MENU_DOWN+1,
	0x22,0xC8,BIGTILE_MENU_DOWN+2,
	0x22,0xC9,BIGTILE_MENU_DOWN+3,
	// tile2
	0x22,0xB6,BIGTILE_MENU_DOWN,
	0x22,0xB7,BIGTILE_MENU_DOWN+1,
	0x22,0xD6,BIGTILE_MENU_DOWN+2,
	0x22,0xD7,BIGTILE_MENU_DOWN+3,
	// header line 1
	0x21,0x2A,0,
	0x21,0x2B,0,
	0x21,0x2C,0,
	0x21,0x2D,0,
	0x21,0x2E,0,
	0x21,0x2F,0,
	// header line 2
	0x21,0x4A,0,
	0x21,0x4B,0,
	0x21,0x4C,0,
	0x21,0x4D,0,
	0x21,0x4E,0,
	0x21,0x4F,0,
	// 13-22 (explanation text)
	0x24,0xBF,0,
	0x24,0xBF,0,
	0x24,0xBF,0,
	0x24,0xBF,0,
	0x24,0xBF,0,
	0x24,0xBF,0,
	0x24,0xBF,0,
	0x24,0xBF,0,
	0x24,0xBF,0,
	0x24,0xBF,0,
};

#define GAME_UPDATE_LIST_COUNT 15
const byte updListData_game[]={
	// timer
	0x20,0x84,0x00,
	0x20,0x85,0x30, // minutes
	0x20,0x86,0x3A, // :
	0x20,0x87,0x30, // seconds
	0x20,0x88,0x30,
	// player position
	0x20,0xC5,0x30,
	0x20,0xC6,0x31, // x
	0x20,0xE5,0x30,
	0x20,0xE6,0x31, // y
	// tile chunk
	0x28,0x00,0,
	0x28,0x00,0,
	0x28,0x00,0,
	0x28,0x00,0,
	// life meter for random mode
	0x28,0x00,0,
	0x28,0x00,0,
};

static byte updList_game[GAME_UPDATE_LIST_COUNT*3];

// internal BS representation, allowing for X and ?s.
#define MAX_INTERNAL_PUZZLE_SIZE	256 // 16x16
static byte gamePuzzleData[MAX_INTERNAL_PUZZLE_SIZE];
// but not all of it is always used...
/* <current setup>
 * 0x00: up; 0x01: down; 0x02: X; 0x03: dot.
 * if (value&0x04)==0x04, then tile is unswappable.
 * (only makes sense for down and X tiles.) */
/* <future planning>
 * tiles can have these states: up,down,x,?/dot.
 * Tiles can also be marked as un-changeable by (value|0x04).
 * We only need four bits for the tile state:
 * 000=up, 001=down, 010=x, 011=?/dot
 * for unswappable tiles, we'd just check the third bit as normal.
 * this can fit a 16x16 puzzle in 128 bytes vs. the current 256 bytes. */

#define MAX_INTERNAL_SOLUTION_SIZE	256 // 16x16
/* solution for currently loaded puzzle.
 * Values come from ROM (Normal Mode; see puzzleData pointers) or Randgen.
 * MAX_INTERNAL_SOLUTION_SIZE is temp because I need to think about it some more.
 * Ideally, this would just use one bit per tile. A 16x16 puzzle would only
 * require 32 bytes vs. the 256 bytes it uses now. */
static byte gameCurrentPuzzleSol[MAX_INTERNAL_SOLUTION_SIZE];

// used to store the byte values of the randomly generated level.
// this currently used for debugging, but is the solution to my problem above.
#define MAX_LEVEL_BYTES 32
static byte randPuzzleBytes[MAX_LEVEL_BYTES];

// used for drawing Rows/Columns of numbers. 7 is the max amount in both dirs.
static byte solDrawArray[7];

// cheat codes
//static unsigned char codeSequence,lastInput,codeComplete;
//static unsigned char codeInputs[x]; // todo: find out max number of inputs needed for codes

// title menu codes
//static const byte codeSabu[5]={ PAD_SELECT,PAD_A,PAD_B,PAD_UP,0 };
//static const byte codeKonami[11]={ PAD_UP,PAD_UP,PAD_DOWN,PAD_DOWN,PAD_LEFT,PAD_RIGHT,PAD_LEFT,PAD_RIGHT,PAD_B,PAD_A,0 };

// zero page vars
#include "zp.h"

// (optionally) turns off the PPU, clears OAM, resets scroll and vram update.
void __fastcall__ reset_vram(unsigned char bDisabled){
	if(bDisabled&1) ppu_off();

	oam_clear();
	set_vram_update(0,0);
	scroll(0,0);
}

// overwrites everything in the master update list with 0x00
void __fastcall__ resetMasterUpdateList(void){
	memfill(masterUpdList,0x00,MAX_MASTER_UPDATE_LIST_COUNT*3);
}

// stoled from shiru and then edited for widths of 5
void __fastcall__ printNum(unsigned int addr, unsigned int num, unsigned char len){
	vram_adr(addr);

	if(len>4) vram_put(0x30+(num/10000%10));
	if(len>3) vram_put(0x30+(num/1000%10));
	if(len>2) vram_put(0x30+(num/100%10));
	if(len>1) vram_put(0x30+(num/10%10));
	vram_put(0x30+(num%10));
}

// also stoled from shiru
void __fastcall__ pal_fade_to(byte to, byte fDelay){
	while(bright!=to){
		rand8();
		delay(fDelay);
		if(bright<to) ++bright; else --bright;
		pal_bright(bright);
	}

	if(!bright){
		ppu_off();
		set_vram_update(0,0);
		scroll(0,0);
	}
}

// clear out all the sprite tiles.
// oam_clear() doesn't do this for whatever reason.
void __fastcall__ clearOamTiles(void){
	for(i=0; i<64; i++){
		*(byte*)(0x0201+(i*4)) = 0;
	}
}

/** Pre-Release Screen ********************************************************/
void prerelScreen(void){
	cnrom_chrbank(0);
	resetMasterUpdateList();
	pal_bg(palBG_PreRel);
	pal_spr(palSPR_Sprite1);
	unrle_vram(nam_prerelease,0x2000);
	wait=96; n=0;

	// write version info
	{
		vram_write((unsigned char *)verNum,0x2296,sizeof(verNum));
		vram_write((unsigned char *)verDate,0x22a8,sizeof(verDate));
		vram_write((unsigned char *)verTime,0x22b3,sizeof(verTime));
		vram_write((unsigned char *)verVariant,0x22C2+(14-(sizeof(verVariant)/2)),sizeof(verVariant));
	}

	set_vram_update(MAX_MASTER_UPDATE_LIST_COUNT,masterUpdList);

	ppu_on_all();
	pal_fade_to(4,3);

	while(1){
		if(!wait&&n==0){
			// show "press start"
			for(i=0;i<sizeof(prerel_PressStart);i++){
				masterUpdList[0+(i*3)] = 0x23;
				masterUpdList[1+(i*3)] = 0x44+i;
				masterUpdList[2+(i*3)] = prerel_PressStart[i];
			}
			n=1;
		}

		ppu_waitnmi();
		++frameCount;
		rand8();

		pad=pad_trigger(0);
		if(pad&PAD_START && wait==0){
			curScreen=0;
			break;
		}

		if(wait) --wait;

		// fade "press start" text
		if(!wait && (n>=1 && n<5)){
			// fade palette in for bgpal 5-7
			for(i=0;i<3;i++){
				pal_col((i+5),palBG_PreRelStart[((n-1)*3)+i]);
			}
			++n;
			ppu_waitnmi();
		}
	}
	// prepare next screen
	pal_fade_to(0,4);
	reset_vram(1);
}

/** Title Screen **************************************************************/
void titleScreen(void){
	cnrom_chrbank(2);
	resetMasterUpdateList();
	pal_bg(palBG_Title);
	pal_spr(palSPR_SpriteTitle);
	unrle_vram(nam_title,0x2000);
	wait=16;
	k=0; m=0; r=0; u=0; v=0; j=0; pad=0;

	set_vram_update(MAX_MASTER_UPDATE_LIST_COUNT,masterUpdList);

	// build information
	vram_write((unsigned char *)verNum,0x2346,sizeof(verNum));
	vram_write((unsigned char *)verDate,0x234b,sizeof(verDate));
	vram_write((unsigned char *)verTime,0x2356,sizeof(verTime));

	ppu_on_all();
	if(bright<4){ pal_fade_to(4,4); }

	while(1){
		// sprites
		curSprite=24;
		// "y"/"s" fix part 1
		curSprite-=8;
		curSprite=oam_spr( (192), (63), 0xE0, 1, curSprite );
		curSprite-=8;
		curSprite=oam_spr( (200), (63), 0xE1, 1, curSprite );
		// "y"/"s" fix part 2
		curSprite-=8;
		curSprite=oam_spr( (192), (71), 0xF0, 1, curSprite );
		curSprite-=8;
		curSprite=oam_spr( (200), (71), 0xF1, 1, curSprite );

		/*
		if(s==4){ // entered sabu code
			curScreen=0xD8;
			s=0;
			break;
		}
		*/

		rand8();
		ppu_waitnmi();
		++frameCount;

		pad = pad_trigger(0);
		/*
		// check for sabu code
		if(pad!=0){
			if(!(pad&codeSabu[s])){ s=0; }
			else if(pad&codeSabu[s]){++s;}
		}
		*/

		if(pad&PAD_START && wait==0){
			sfx_play(SFX_START,0);
			curScreen=1;
			// I should wait a few frames before fading out here
			break;
		}

		if(wait) --wait;
	}

	// prepare next screen
	pal_fade_to(0,4);
	reset_vram(1);
}

/** Main Menu *****************************************************************/

// update mode explanation; relies on current menu item
void mainMenu_updateExplain(void){
	if(explainLine<5){
		for(i=0;i<10;i++){
			masterUpdList[60+(i*3)] = (explainLine>=3)?0x22:0x21;
			masterUpdList[61+(i*3)] = 0xA5+(0x20*explainLine)+(explainCount-i)+i;
			masterUpdList[62+(i*3)] = (menuMain_ExplainTexts[j])[explainCount+(22*explainLine)];

			++explainCount;

			if(explainCount==MAINMENU_EXPLAIN_LINELEN){
				++explainLine;
				explainCount=0;
			}
		}
	}
}

void mainMenu(void){
	cnrom_chrbank(0);
	resetMasterUpdateList();
	oam_clear();
	clearOamTiles();

	// clear nametables
	vram_adr(0x2000);
	vram_fill(0,0x3C0);
	vram_adr(0x2400);
	vram_fill(0,0x3C0);

	pal_bg(palBG_MainMenu);
	pal_spr(palSPR_Sprite1);

	unrle_vram(nam_menuMain,0x2000);

	ppu_on_all();

	i=0;
	j=0; // menu selection (0=norm, 1=rand)
	k=0;
	m=0; // prev menu selection
	x=0;
	y=0;
	randLevelDiff=0; // reset random mode difficulty

	// explanation stuffs
	explainCount=0;
	explainLine=0;

	// setup update list
	memcpy(masterUpdList,updListData_menuMain,sizeof(updListData_menuMain));
	set_vram_update(MAIN_MENU_UPDATE_LIST_COUNT,masterUpdList);

	if(bright<4){ pal_fade_to(4,4); }

	if(gameMode>1){ gameMode=1; }
	if(gameMode!=0){ j=gameMode; }

	// set initial cursor x position so the cursor doesn't jump
	cursorX=(j==1)?144:32;

	while(1){
		if(frameCount%8==0){
			++q;
			if(q>3){ q=q%4; }
		}

		// add 16px tile cursor
		curSprite=16;
		for(i=0;i<4;++i){
			x=(i%2)?8:0;
			y=(i>1)?8:0;
			curSprite=oam_spr(
				(32+x)+(cursorX), (167+y),
				bigCursorFrames[q], chrTest_cAttr[i], curSprite
			);
			curSprite-=8;
		}

		rand8();
		ppu_waitnmi();
		++frameCount;

		// 0x23E9.4 - normal
		// 0x23EE.6 - random

		// 42,43,44-old selection attribute
		// update old selection
		if(m!=j){
			masterUpdList[0]=0x22;
			masterUpdList[3]=0x22;
			masterUpdList[6]=0x22;
			masterUpdList[9]=0x22;
			//masterUpdList[42]=0x23;
			switch(m){
				case 0:
					masterUpdList[1]=0xA8;
					masterUpdList[4]=0xA9;
					masterUpdList[7]=0xC8;
					masterUpdList[10]=0xC9;

					//masterUpdList[43]=0xE9;
					//masterUpdList[44]=0x00;
					break;
				case 1:
					masterUpdList[1]=0xB6;
					masterUpdList[4]=0xB7;
					masterUpdList[7]=0xD6;
					masterUpdList[10]=0xD7;

					//masterUpdList[43]=0xEE;
					//masterUpdList[44]=0x00;
					break;
				default: break;
			}
			masterUpdList[2]=BIGTILE_MENU_DOWN;
			masterUpdList[5]=BIGTILE_MENU_DOWN+1;
			masterUpdList[8]=BIGTILE_MENU_DOWN+2;
			masterUpdList[11]=BIGTILE_MENU_DOWN+3;
		}

		// 45,46,47-old selection attribute
		// update new selection
		masterUpdList[12]=0x22;
		masterUpdList[15]=0x22;
		masterUpdList[18]=0x22;
		masterUpdList[21]=0x22;
		//masterUpdList[45]=0x23;
		switch(j){
			case 0:
				masterUpdList[13]=0xA8;
				masterUpdList[16]=0xA9;
				masterUpdList[19]=0xC8;
				masterUpdList[22]=0xC9;

				//masterUpdList[46]=0xE9;
				//masterUpdList[47]=(3<<4);
				break;
			case 1:
				masterUpdList[13]=0xB6;
				masterUpdList[16]=0xB7;
				masterUpdList[19]=0xD6;
				masterUpdList[22]=0xD7;

				//masterUpdList[46]=0xEE;
				//masterUpdList[47]=(3<<6);
				break;
			default: break;
		}
		masterUpdList[14]=(j==1)?BIGTILE_MENU_DOT:BIGTILE_MENU_UP;
		masterUpdList[17]=(j==1)?BIGTILE_MENU_DOT+1:BIGTILE_MENU_UP+1;
		masterUpdList[20]=(j==1)?BIGTILE_MENU_DOT+2:BIGTILE_MENU_UP+2;
		masterUpdList[23]=(j==1)?BIGTILE_MENU_DOT+3:BIGTILE_MENU_UP+3;

		// update mode header
		// line 1
		for(i=0;i<MAINMENU_HEADER_LENGTH;i++){
			masterUpdList[26+(i*3)] = (menuMain_HeaderTops[j])[i];
		}
		// line 2
		for(i=0;i<MAINMENU_HEADER_LENGTH;i++){
			masterUpdList[44+(i*3)] = (menuMain_HeaderBots[j])[i];
		}

		// and now how the hell am I supposed to update 22*5 tiles
		// for the explanation
		mainMenu_updateExplain();

		// handle input and such
		pad=pad_trigger(0);
		if(pad&PAD_LEFT){
			if(j>0){
				m=j; --j;
				sfx_play(SFX_MAINMENUCHANGE,0);
				explainCount=0;
				explainLine=0;
			}
		}
		if(pad&PAD_RIGHT){
			if(j<1){
				m=j; ++j;
				sfx_play(SFX_MAINMENUCHANGE,0);
				explainCount=0;
				explainLine=0;
			}
		}
		cursorX=(j==1)?144:32; // update cursor X pos

		// show credits on select button
		if(pad&PAD_SELECT){
			sfx_play(SFX_START,1);
			curScreen=0xFF;
			break;
		}

		if(pad&PAD_A || pad&PAD_START){
			sfx_play(SFX_MAINMENUSELECT,1);
			wait=20;
			while(wait>0){
				rand8();
				--wait;
			}

			// move on to the next screen based on j
			gameMode=j; // the game mode maps perfectly.
			curScreen=(j==1)?3:2;
			break;
		}
	}

	// prepare next screen
	pal_fade_to(0,3);
}

/** Normal Mode Menu **********************************************************/
/* The way the VRAM updates are handled are kind of fucked up, in my eyes.
 * Granted, this is from future me, whereas the code was written by past me.
 * Future me took enough advice from this section to not fuck it up in
 * Random v3's menu, but now I have to go back and document Normal Mode.
 *
 * And as it turns out, it wasn't as bad as I had expected, especially to
 * include the proper loading... It just needs testing.
 */

// update level list (0x96); offsets given are for the first line.
void normalMenu_UpdateLevelList(void){
	// 10 lines in total (0-9)
	if(w<10){
		// status (0x2105)
		q=*(byte*)(SRAM_NORMALDATA + (4 * ((pageNum*10)+w)))>>6;
		k16=0x2105+(w*0x20);
		masterUpdList[0] = k16>>8;
		masterUpdList[1] = (k16&0xFF);
		switch(q){
			case 0: j=0x8A; break;	// no play
			case 1: j=0x89; break;	// failed
			case 2: j=0x88; break;	// passed
			//case 3: j=0x2A; break;	// perfect clear w/life meter
			default: j=0x8A; break;
		}
		masterUpdList[2] = j;

		// number (0x2107)
		k16=0x2107+(w*0x20);
		j=0;
		for(i=0;i<3;i++){
			masterUpdList[3+(i*3)] = k16>>8;
			masterUpdList[4+(i*3)] = (k16&0xFF)+i;
			switch(i){
				case 0: j=(((pageNum*10)+w)+1)/100%10; break;
				case 1: j=(((pageNum*10)+w)+1)/10%10; break;
				case 2: j=(((pageNum*10)+w)+1)%10; break;
				default: break;
			}
			masterUpdList[5+(i*3)] = 0x30+j;
		}

		// size (0x210D-0x22111)
		// broken on anything over 9x9?
		k16=0x210D+(w*0x20);
		x=(puzzleData[(pageNum*10)+w])[0]&0x0F;
		for(i=0;i<4;i++){
			masterUpdList[12+(i*3)] = k16>>8;
			j=(k16&0xFF)+((i>1)?1:0);
			masterUpdList[13+(i*3)] = j+i;

			if(i==1||i==3){
				// ones digit
				if(5+x > 9){
					masterUpdList[14+(i*3)] = 0x30+((5+x)-10);
				}
				else{
					masterUpdList[14+(i*3)] = 0x30+5+x;
				}
			}
			else {
				// tens digit
				masterUpdList[14+(i*3)] = (x>4)?0x31:0x30;
			}
		}

		// difficulty (0x2115)
		k16=0x2115+(w*0x20);
		x=(puzzleData[(pageNum*10)+w])[0]>>4;
		for(i=0;i<6;i++){
			masterUpdList[27+(i*3)] = k16>>8;
			masterUpdList[28+(i*3)] = (k16&0xFF)+i;
			masterUpdList[29+(i*3)] = (menuNormal_Difficulty[x])[i];
		}

		++w;

		// next line
		v-=0x0F;
	}
}

// update level information (0x0F)
// u is decremented after running each segment.
void normalMenu_UpdateLevelInfo(void){
	// Segment 0x0F: Update level number and name
	if(u==0x0F){
		g=m+1;
		// number (0x2267)
		for(i=0;i<3;i++){
			masterUpdList[0+(i*3)] = 0x22;
			masterUpdList[1+(i*3)] = 0x67+i;
			switch(i){
				case 0:
					if(g==0){ j=0x32; }
					else{ j=(g>99)?(g/100%10):0; }
					break;
				case 1:
					if(g==0){ j=0x35; }
					else{ j=(g>9)?(g/10%10):0; }
					break;
				case 2:
					if(g==0){ j=0x36; }
					else{ j=(g%10); }
					break;
				default: break;
			}
			masterUpdList[2+(i*3)] = 0x30+j;
		}
		// name (0x226C)
		q=(*(byte*)(SRAM_NORMALDATA+(4*m))&0xC0)>>6;
		if(q==2){
			// real name
			j=0; z=0;
			for(i=0;i<16;i++){
				j=(puzzleNames[m])[i];
				if(j==0xFF){ z=i; }
				masterUpdList[9+(i*3)] = 0x22;
				masterUpdList[10+(i*3)] = 0x6C+i;
				masterUpdList[11+(i*3)] = (z==0)?j:0;
			}
		}
		else{
			// hidden name
			for(i=0;i<16;i++){
				masterUpdList[9+(i*3)] = 0x22;
				masterUpdList[10+(i*3)] = 0x6C+i;
				masterUpdList[11+(i*3)] = 0x3F;
			}
		}

		u-=0x02;
	}

	// Segment 0x0D: Update level difficulty and size
	else if(u==0x0D){
		// difficulty (0x22B7);
		j=((puzzleData[m])[0])>>4;
		for(i=0;i<6;i++){
			masterUpdList[0+(i*3)] = 0x22;
			masterUpdList[1+(i*3)] = 0xB7+i;
			masterUpdList[2+(i*3)] = (menuNormal_Difficulty[j])[i];
		}

		// size (0x22A9)
		// xxx: only confirmed working for 05x05, 08x08.
		// 16x16 is broken
		j=((puzzleData[m])[0])&0x0F;
		for(i=0;i<4;i++){
			masterUpdList[18+(i*3)]=0x22;
			masterUpdList[19+(i*3)]=((i>1)?0xAA:0xA9)+i;

			if(i==1||i==3){
				// ones digit
				if(j+5 > 9){
					masterUpdList[20+(i*3)]=0x30+((5+j)-10);
				}
				else{
					masterUpdList[20+(i*3)]=0x30+5+j;
				}
			}
			else{
				// tens digit
				masterUpdList[20+(i*3)]=(j>4)?0x31:0x30;
			}
		}
		u-=0x02;
	}

	// Segment 0x0B: Update number of plays, status, and best time
	else if(u==0x0B){
		// plays (0x22EB)
		q=(*(byte*)(SRAM_NORMALDATA+(4*m)+2));
		for(i=0;i<3;i++){
			masterUpdList[0+(i*3)] = 0x22;
			masterUpdList[1+(i*3)] = 0xEB+i;
			switch(i){
				case 0: j=(q/100%10); break;
				case 1: j=(q/10%10); break;
				case 2: j=(q%10); break;
				default: break;
			}
			masterUpdList[2+(i*3)] = 0x30+j;
		}

		// status (0x2264)
		q=*(byte*)(SRAM_NORMALDATA+(4*m))>>6;
		j=0;
		masterUpdList[9] = 0x22;
		masterUpdList[10] = 0x64;
		switch(q){
			case 0: j=0x8A; break;	// no play
			case 1: j=0x89; break;	// failed
			case 2: j=0x88; break;	// passed
			//case 3: j=0x2A; break;	// perfect clear w/life meter
			default: j=0x8A; break;
		}
		masterUpdList[11] = j;

		// best time (0x22F7)
		g=(*(byte*)(SRAM_NORMALDATA+(4*m))&0x3F);			// minutes
		q=(*(byte*)(SRAM_NORMALDATA+(4*m)+1)&0xFC)>>2;	// seconds
		j=0;
		for(i=0;i<4;i++){
			masterUpdList[12+(i*3)]=0x22;
			masterUpdList[13+(i*3)]=((i>1)?0xF8:0xF7)+i;
			switch(i){
				case 0: j=(g/10%10)+0x30; break;
				case 1: j=(g%10)+0x30; break;
				case 2: j=(q/10%10)+0x30; break;
				case 3: j=(q%10)+0x30; break;
				default: j=0x30; break;
			}
			masterUpdList[14+(i*3)]=j;
		}
		u-=0x03;
	}

	// anything else means we're done with the update.
	// this currently doesn't pose a problem, but could in the future.
	else{ u=0; }
}

void normalMenu_updateLifeMeterText(void){
	// strings are 14 characters long and drawn to 0x2369-0x2376.
	// masterUpdList index starts from 48.
	if(enableLifeMeter==1){
		// draw menuNormal_LifeMeter
		for(i=0;i<14;i++){
			masterUpdList[48+(i*3)]=0x23;
			masterUpdList[49+(i*3)]=0x69+i;
			masterUpdList[50+(i*3)]=menuNormal_LifeMeter[i];
		}
	}
	else if(enableLifeMeter == 0){
		// draw blanks
		for(i=0;i<14;i++){
			masterUpdList[48+(i*3)]=0x23;
			masterUpdList[49+(i*3)]=0x69+i;
			masterUpdList[50+(i*3)]=0x00;
		}
	}

	z=0; // aaaaand we're done.
}

void normalMenu_updatePageNumberText(void){
	// two characters, drawn to 0x2332 and 0x2333.

	// tens digit
	masterUpdList[48]=0x23;
	masterUpdList[49]=0x32;
	masterUpdList[50]=((pageNum+1)/10%10)+0x30;
	// ones digit
	masterUpdList[51]=0x23;
	masterUpdList[52]=0x33;
	masterUpdList[53]=((pageNum+1)%10)+0x30;
}

// calculate scrollbar position (0-255 ~ 0-70 fixY)
// xxx: should I really be using division here, or should I use a table of
// precalculated values? (see tools/scrollbarVal.txt)
void normalMenu_scrollBarPos(void){ s = (m*70)/255; }

#define LAST_LEVEL_PAGE		1

void normalMenu(void){
	cnrom_chrbank(0);
	resetMasterUpdateList();
	oam_clear();
	clearOamTiles();
	pal_bg(palBG_menuNormal);
	pal_spr(palSPR_Sprite1);

	unrle_vram(nam_menuNormal3,0x2000);

	pad=0;
	i=0;
	j=0;
	k=0;		// cursor position (0-9)
	m=0;		// real selection (0-255)
	n=0;
	p=0;		// was level list page number
	r=0;		// drawing result.
	s=0;		// scrollbar position (0-255 ~ 108-178 Y, 0-70 fixY)
	u=0x0F;		// update level info drawing
	v=0x96;		// update level list drawing (0x96 = update 10 lines)
	w=0;
	z=0;		// update life meter text drawing

	// may want repeating up/down input here?
	repeatDir=0;

	wait=24;

	// set level and cursor as needed
	if(gameLevel!=0){
		m=gameLevel;	// set proper level selection
		k=gameLevel%10;
	}

	// write initial values to vram

	// initial page number (0x2332)
	// the actual number itself has been determined above;
	// a value of 1 is added for display purposes.
	printNum(0x2332,pageNum+1,2);

	set_vram_update(MAIN_MENU_UPDATE_LIST_COUNT,masterUpdList);

	ppu_on_all();

	if(bright<4){ pal_fade_to(4,4); }

	while(1){
		// draw cursor
		curSprite=1<<3;
		curSprite=oam_spr(31,63+((m%10)*8),SPR_ARROW_RIGHT,2,curSprite);

		// draw scrollbar tick (y: 108 min, 178 max)
		curSprite=oam_spr(232,63+s,MENU_DOT,1,curSprite);

		if(z>0){ normalMenu_updateLifeMeterText(); }
		else if(w>0){ normalMenu_updatePageNumberText(); }

		if(u>0){ normalMenu_UpdateLevelInfo(); }
		else if(v>0){ normalMenu_UpdateLevelList(); }

		ppu_waitnmi();
		frameCount++;
		rand8();

		if(wait){
			--wait;
			continue;
		}

		pad=pad_trigger(0);

		// Select Button: Toggle Life Meter
		if(pad&PAD_SELECT){
			sfx_play(SFX_MAINMENUCHANGE,2);
			enableLifeMeter^=1;		// toggle life meter
			z=1;
		}

		// A Button/Start: Begin game
		if(pad&PAD_A || pad&PAD_START){
			curScreen=4;
			gameLevel=m;
			gameMode=0;
			break;
		}
		// B Button: Cancel
		if(pad&PAD_B){
			sfx_play(SFX_MENUBACK,1);
			curScreen=1;
			gameLevel=0;		// reset level
			enableLifeMeter=0;	// reset life meter
			pageNum=0;			// reset page number
			break;
		}

		// up and down are supposed to wrap around the list.
		// xxx: this needs to take pages into account (aka: not use m).
		if(pad&PAD_UP){
			// decrease cursor
			if(k>0){ --k; }
			// wrap cursor
			else{ k=9; }

			m=(pageNum*10)+k;
			u=0x0F;			// level info update
			sfx_play(SFX_MENUCHANGEUP,0);
		}
		if(pad&PAD_DOWN){
			// increase cursor
			if(k<9){ ++k; }
			// wrap cursor
			else{ k=0; }

			m=(pageNum*10)+k;
			u=0x0F;			// level info update
			sfx_play(SFX_MENUCHANGEDOWN,0);
		}

		// left and right switch pages of 10 levels each.
		if(pad&PAD_LEFT){
			// go back a page

			// wrap
			if(pageNum==0){ pageNum=LAST_LEVEL_PAGE; }
			// decrement
			else{ --pageNum; }

			m=(pageNum*10)+k;
			v=0x96;			// page info update
			w=0;			// reset bullshit
			u=0x0F;			// level info update
		}
		if(pad&PAD_RIGHT){
			// go forward a page

			// wrap
			if(pageNum==LAST_LEVEL_PAGE){ pageNum=0; }
			// increment
			else{ ++pageNum; }

			m=(pageNum*10)+k;
			v=0x96;			// page info update
			w=0;			// reset bullshit
			u=0x0F;			// level info update
			normalMenu_updatePageNumberText();
		}

		normalMenu_scrollBarPos();
	}

	// prepare next screen
	pal_fade_to(0,3);
}

/** Random3 (Random Mode Menu v3) code ****************************************/
// game type	single/set
// life meter	on/off

/*--Single options-----------------------*/
// Size
// Skill
// Use Seed? y/n
// Seed: 00000-65535

/*--Set options--------------------------*/
// Number of Puzzles (000-999)
// Base Size
// Base Skill

/*---- Level Config ----*/
// This part can be left for later, as it's a bit more complex.
// level number
// size
// skill

// how u (the update variable) works:
/* 00000000
 * xABCDEFL
 * x - currently unused
 * A - (Normal) Size		(Set) Num. Puzzles
 * B - (Normal) difficulty	(Set) base size
 * C - (Normal) use seed	(Set) base difficulty
 * D - (Normal) seed		(Set) level config
 * E - (Normal) unused		(Set) level size
 * F - (Normal) unused		(Set) level difficulty
 * L - Life Meter
 * 
 * Updating Num. Puzzles (A in Set mode) implicitly modifies the maximum
 * number in Level Config (D in Set mode). Whether or not the currently edited
 * level number needs to change (if max number of puzzles is decreased to a
 * number below the current level) needs to be handled as well.
 */

void randomMenu_UpdateBGTiles(){
	if(u&1){
		// life meter [0-11] (change on both nametables)

		// on (0x2151;0x2551)
		masterUpdList[ 0]=0x21;
		masterUpdList[ 1]=0x51;
		masterUpdList[ 2]=(enableLifeMeter==1)?0xC8:0xC9;
		/*******************/
		masterUpdList[ 3]=0x25;
		masterUpdList[ 4]=0x51;
		masterUpdList[ 5]=(enableLifeMeter==1)?0xC8:0xC9;

		// off (0x2157;0x2557)
		masterUpdList[ 6]=0x21;
		masterUpdList[ 7]=0x57;
		masterUpdList[ 8]=(enableLifeMeter==0)?0xC8:0xC9;
		/*******************/
		masterUpdList[ 9]=0x25;
		masterUpdList[10]=0x57;
		masterUpdList[11]=(enableLifeMeter==0)?0xC8:0xC9;

		u^=1;
	}

	if(n==0){
		// random single [12-xx]
		if(u&2){
			// size (4 tiles)
			masterUpdList[12]=0x21;
			masterUpdList[13]=0xCF;
			masterUpdList[14]=(randLevelSize>4)?0x31:0x30;
			/*---------------*/
			masterUpdList[15]=0x21;
			masterUpdList[16]=0xD0;
			masterUpdList[17]=(randLevelSize>4)?0x30+(randLevelSize-5):0x30+5+randLevelSize;
			/*****************/
			masterUpdList[18]=0x21;
			masterUpdList[19]=0xD2;
			masterUpdList[20]=(randLevelSize>4)?0x31:0x30;
			/*---------------*/
			masterUpdList[21]=0x21;
			masterUpdList[22]=0xD3;
			masterUpdList[23]=(randLevelSize>4)?0x30+(randLevelSize-5):0x30+5+randLevelSize;

			u^=2;
		}
		if(u&4){
			// difficulty (6 tiles)
			// xxx: determine difficulty

			masterUpdList[24]=0x22;
			masterUpdList[25]=0x0F;
			masterUpdList[26]=0x89;
			/*---------------*/
			masterUpdList[27]=0x22;
			masterUpdList[28]=0x10;
			masterUpdList[29]=0x89;
			/*---------------*/
			masterUpdList[30]=0x22;
			masterUpdList[31]=0x11;
			masterUpdList[32]=0x89;
			/*---------------*/
			masterUpdList[33]=0x22;
			masterUpdList[34]=0x12;
			masterUpdList[35]=0x89;
			/*---------------*/
			masterUpdList[36]=0x22;
			masterUpdList[37]=0x13;
			masterUpdList[38]=0x89;
			/*---------------*/
			masterUpdList[39]=0x22;
			masterUpdList[40]=0x14;
			masterUpdList[41]=0x89;

			u^=4;
		}
		if(u&8){
			// use seed (2 tiles)
			// on (0x2151;0x2551)
			masterUpdList[42]=0x22;
			masterUpdList[43]=0x4F;
			masterUpdList[44]=(useSeed==1)?0xC8:0xC9;

			// off (0x2157;0x2557)
			masterUpdList[45]=0x22;
			masterUpdList[46]=0x55;
			masterUpdList[47]=(useSeed==0)?0xC8:0xC9;

			u^=8;
		}
		if(u&16){
			// seed number (5 tiles)
			masterUpdList[48]=0x22;
			masterUpdList[49]=0x8F;
			masterUpdList[50]=0x30;
			/*---------------*/
			masterUpdList[51]=0x22;
			masterUpdList[52]=0x90;
			masterUpdList[53]=0x30;
			/*---------------*/
			masterUpdList[54]=0x22;
			masterUpdList[55]=0x91;
			masterUpdList[56]=0x30;
			/*---------------*/
			masterUpdList[57]=0x22;
			masterUpdList[58]=0x92;
			masterUpdList[59]=0x30;
			/*---------------*/
			masterUpdList[60]=0x22;
			masterUpdList[61]=0x93;
			masterUpdList[62]=0x30;

			u^=16;
		}
	}
	else if(n==1){
		// random set [12-xx]
		if(u&2){
			// num puzzles (3 tiles)
		}
		if(u&4){
			// base size (4 tiles)
		}
		if(u&8){
			// base difficulty (6 tiles)
		}
		if(u&16){
			// level config (e.g. "000/000"; 6 tiles)
		}
		if(u&32){
			// level size (4 tiles)
		}
		if(u&64){
			// level difficulty (6 tiles)
		}
	}
}

static const byte randSingleCursorY[6] = { 63, 79, 111, 127, 143, 159 };
//static const byte randSetCursorY[8] = { 63, 79, xx,xx,xx,xx,xx,xx };

void randomMenu(void){
	cnrom_chrbank(0);
	resetMasterUpdateList();
	oam_clear();
	clearOamTiles();
	pal_bg(palBG_menuRandom);
	pal_spr(palSPR_Sprite1);

	unrle_vram(nam_RandomSingle,0x2000);
	unrle_vram(nam_RandomSet,0x2400);

	pad=0;
	i=0;
	j=0;
	k=0;
	m=0;		// menu cursor (0,1: top part; 2-5: bottom single; 2-7: bottom set)
	n=0;		// random mode type (0=single,1=set)
	s=0;		// seed cursor (0-4)
	u=0xFF;		// display update

	// xxx: same count as main menu?
	set_vram_update(MAIN_MENU_UPDATE_LIST_COUNT,masterUpdList);
	ppu_on_all();
	if(bright<4){ pal_fade_to(4,4); }
	wait=24;

	while(1){
		// sprites
		// * option cursor
		curSprite = 1<<2;
		curSprite=oam_spr(23,randSingleCursorY[m],SPR_ARROW_RIGHT,2,curSprite);

		// bg tiles
		if(u>0){
			// run bg tiles update
			randomMenu_UpdateBGTiles();
		}

		ppu_waitnmi();
		frameCount++;
		rand8();

		if(wait){ --wait; rand8(); continue; }

		// handle input
		pad=pad_trigger(0);

		// Start = begin game
		if(pad & PAD_START){
			curScreen=4;
			// check n (0=random normal [gameMode=1], 1=random set [gameMode=2])
			gameMode=n+1;
			break;
		}

		// everything else MAY depend on the state of this menu.
		// still thinking about it.

		// B = cancel
		if(pad&PAD_B){
			sfx_play(SFX_MENUBACK,1);
			curScreen=1;
			gameMode=1;
			randLevelDiff=0;
			enableLifeMeter=0;
			break;
		}

		// now for things that depend on other shit!
		// Left/Right do different things based on current option!
		// A Button depends on current selection.
		// * <Single> 5/Seed: press A to toggle seed editor

		if(m==0){
			// swap modes, update scroll (disabled for demo2)
			if(pad&PAD_LEFT || pad&PAD_RIGHT){
				/*
				gameMode = (gameMode == 1)?2:1;
				scroll((gameMode == 2)?256:0, 0);
				*/
			}
		}
		else if(m==1){
			// toggle life meter
			if(pad&PAD_LEFT || pad&PAD_RIGHT){
				enableLifeMeter^=1;
				u|=1;	// life meter vram update
				sfx_play(SFX_MENUCHANGEOPTION,1);
			}
		}
		else{
			if(n==0){
				// random single
				if(m==2){
					// puzzle size
					if(pad&PAD_LEFT){
						//if(randLevelSize == 0){ randLevelSize = 0x0B; }

						// XXX HACK: 16x16 doesn't work right now
						if(randLevelSize == 0){ randLevelSize = 0x0A; }
						else{ randLevelSize -= 1; }
						u|=2;
						sfx_play(SFX_MENUCHANGEOPTION,1);
					}
					else if(pad&PAD_RIGHT){
						// size up (wrap to 5x5 if needed)
						//if(randLevelSize == 0x0B){ randLevelSize = 0; }

						// XXX HACK: 16x16 doesn't work right now
						if(randLevelSize == 0x0A){ randLevelSize &= 0x07; } // 2015 edit, 2014 version set to 0
						else{ randLevelSize += 1; }
						u|=2;
						sfx_play(SFX_MENUCHANGEOPTION,1);
					}
					
				}
				else if(m==3){
					// puzzle difficulty
					if(pad&PAD_LEFT){
						// difficulty down
						if(randLevelDiff == 0){ randLevelDiff = 4; }
						else{ randLevelDiff -= 1; }
					}
					else if(pad&PAD_RIGHT){
						if(randLevelDiff == 4){ randLevelDiff = 0; }
						else{ randLevelDiff += 1; }
					}
				}
				else if(m==4){
					// use seed
					if(pad&PAD_LEFT || pad&PAD_RIGHT){
						useSeed^=1;
						u|=8;
						sfx_play(SFX_MENUCHANGEOPTION,1);
					}
				}
				else if(m==5){
					// seed
					// update levelSeed
				}
			}
			else if(n==1){
				// random set (not implemented for demo2)
			}
		}

		// up/down wraparound depends on current mode
		if(pad&PAD_UP){
			if(m==0){
				// max item depends on current mode
				//m=(n==1)?7:5;

				// demo version: can't change difficulty or seed
				m=(n==1)?7:2;
			}
			else{ m-=1; }
			sfx_play(SFX_MENUCHANGEUP,0);
		}

		if(pad&PAD_DOWN){
			// max depends on current mode

			// random single
			//if(n==0 && m==5){ m=0; }
			// random single (demo)
			if(n==0 && m==2){ m=0; }

			// random set
			else if(n==1 && m==7){ m=0; }
			// otherwise
			else{ m+=1; }
			sfx_play(SFX_MENUCHANGEDOWN,0);
		}
	}

	// ok time to see what we're gonna play.
	if(n==0){
		// random single mode

		// use seed if requested
		if(useSeed==1){
			// set_rand(levelSeed);
		}
	}
	else if (n==1){
		// random set mode
		gameLevel=0;	// first level
	}

	pal_fade_to(0,3);
}

/** Result Screens ************************************************************/
// handles normal mode (should also handle random single)
/* todo: what changes need to be made for random single?
 * replace level name			[difficulty] [levelsize] (e.g. "Master 16x16")
 * replace level number			unsure of replacement
 * replace record text			Level Seed
 */

/*-- Normal Mode & Random Single --*/
void resultScreen_Normal(void){
	// check game mode; we don't need to mess with SRAM in Random Single.
	if(gameMode==1){
		// Random Single Mode

		// overwrite the "Level 000" text

		// 0x22C2: 0x8C,0xC9,0xCE,0xC8,0xCE
		vram_adr(0x22C2);
		vram_put(0x8C);
		vram_put(0xC9);
		vram_put(0xCE);
		vram_put(0xC8);
		vram_put(0xCE);

		// 0x22E3: 0x90,0x90,0x9C,0,0,0,0
		vram_adr(0x22E3);
		vram_put(0x90);
		vram_put(0x90);
		vram_put(0x9C);
		vram_put(0);
		vram_put(0);
		vram_put(0);
		vram_put(0);
	}
	else if(gameMode==0){
		// Normal Mode

		// sram-related things
		// check for status
		s=(*(byte*)(SRAM_NORMALDATA+(4*gameLevel))&0xC0)>>6;
		if(s!=2){
			// not already cleared, clear level.
			*(byte*)(SRAM_NORMALDATA+(4*gameLevel))|=0x80;
		}

		// check for record time
		m=(*(byte*)(SRAM_NORMALDATA+(4*gameLevel))&0x3F); // minutes
		t=(*(byte*)(SRAM_NORMALDATA+(4*gameLevel)+1)&0xFC)>>2; // seconds
		t=(m*60)+t;
		n=(min*60)+sec;
		r=0;
		if(n>0){
			if(t==0 || n<t){
				*(byte*)(SRAM_NORMALDATA+(4*gameLevel))=0x80|(min&0x3F); // minutes
				*(byte*)(SRAM_NORMALDATA+(4*gameLevel)+1)=(sec)<<2; // seconds
				r=1;
			}
		}

		// level number (0x22E4)
		printNum(0x22E4,(gameLevel&0xFF)+1,3);

		// level name (0x22EA); 16chars max
		z=0;
		vram_adr(0x22EA);
		for(i=0;i<16;i++){
			j=(puzzleNames[gameLevel])[i];
			if(j==0xFF){ z=i; }
			vram_put((z==0)?j:0);
		}

		// record text (if any) (0x2330)
		if(r!=0){
			vram_write((byte*)result_NewRecord,0x2330,11);
		}
	}

	// xxx: this only handles 5x5 to 8x8 puzzles properly; anything higher
	// is currently untested.

	// 15x15 is known to be broken.

	// picture display is handled in the while loop for family picross+.
	// however, for this version, I'm lazy as shit and want to do it here.

	/* Puzzle size information:
	 * Size____| Begin	| End Row1	| End Tile	| Tile Size
	 * 5x5		0x212B	| 0x2154	| 0x2254	| 2x2 tiles
	 * 6x6		0x210A	| 0x2135	| 0x2275	| 2x2 tiles
	 * 7x7		0x20E9	| 0x2116	| 0x2296	| 2x2 tiles
	 * 8x8		0x20C8	| 0x20F7	| 0x22B7	| 2x2 tiles
	 * ----------------------------------------------------
	 * 9x9		0x212B	| 0x2133	| 0x2233	| 1x1 tile
	 * 10x10	0x212B	| 0x2134	| 0x2254	| 1x1 tile
	 * 11x11	0x210A	| 0x2114	| 0x2254	| 1x1 tile
	 * 12x12	0x210A	| 0x2115	| 0x2275	| 1x1 tile
	 * 13x13	0x20E9	| 0x20F5	| 0x2275	| 1x1 tile
	 * 14x14	0x20E9	| 0x20F6	| 0x2296	| 1x1 tile
	 * 15x15	0x20C8	| 0x20D6	| 0x2296	| 1x1 tile
	 * 16x16	0x20C8	| 0x20D7	| 0x22B7	| 1x1 tile
	*/

	// set initial position based on puzzle size
	switch(puzzleW){
		case  5: i16=0x212B; break;
		case  6: i16=0x210A; break;
		case  7: i16=0x20E9; break;
		case  8: i16=0x20C8; break;
		case  9: i16=0x212B; break;
		case 10: i16=0x212B; break;
		case 11: i16=0x210A; break;
		case 12: i16=0x210A; break;
		case 13: i16=0x20E9; break;
		case 14: i16=0x20E9; break;
		case 15: i16=0x20C8; break;
		case 16: i16=0x20C8; break;
		default: i16=0x212B; break;
	}

	for(i=0;i<puzzleH;i++){
		// update row position
		if(i>0){
			if(puzzleH<9){i16+=BIGTILE_ROW;}
			else{i16+=ONETILE_ROW;}
		}

		for(j=0;j<puzzleW;j++){
			k=gameCurrentPuzzleSol[(i*puzzleH)+j];
			if(k==1){
				if(puzzleH<9){
					// top
					vram_adr(i16+(j*2));
					for(n=0;n<2;n++){ vram_put(1); };
					// bottom
					vram_adr(i16+0x20+(j*2));
					for(n=0;n<2;n++){ vram_put(1); };
				}
				else{
					// top
					vram_adr(i16+j);
					vram_put(1);
					// bottom
					vram_adr(i16+0x20+j);
					vram_put(1);
				}
			}
			else{
				if(puzzleH<9){
					// top
					vram_adr(i16+(j*2));
					for(n=0;n<2;n++){ vram_put(0); };
					// bottom
					vram_adr(i16+0x20+(j*2));
					for(n=0;n<2;n++){ vram_put(0); };
				}
				else{
					// top
					vram_adr(i16+j);
					vram_put(0);
					// bottom
					vram_adr(i16+0x20+j);
					vram_put(0);
				}
			}
		}
	}

	// time (0x2328)
	printNum(0x2328,min,2); // minutes
	printNum(0x232B,sec,2); // seconds
}

/*-- Random Set --*/
void resultScreen_Random(void){
	// this screen is set up for normal random mode by default.

	// todo: display result_GameOver if failed

	// check for status in random mode (but not infinite random.)
	if(gameMode==1){
		if((gameLevel+1)==(randLevelDiff+1)*10){
			s=(*(byte*)(SRAM_RANDOMDATA+(10*randLevelDiff))&0x80)>>7;
			// set pass flag if not set already.
			if(s!=1){ *(byte*)(SRAM_RANDOMDATA+(10*randLevelDiff))=1<<7; }

			// get time
			t=(*(byte*)(SRAM_RANDOMDATA+(10*randLevelDiff)))&0x3F; // minutes

			// get second
			p=(*(byte*)(SRAM_RANDOMDATA+(10*randLevelDiff)+1))&0x3F; // seconds
			// then add the two up and compare to our time
			t=(t*60)+p;

			u=min*60;
			v=u+sec;
			r=0;
			if(s==0 && t==0){ // no record exists, automatically write time.
				(*(byte*)(SRAM_RANDOMDATA+(10*randLevelDiff)))=(1<<7)|(min&0x3F); // set minute
				(*(byte*)(SRAM_RANDOMDATA+(10*randLevelDiff)+1))=(sec&0x3F); // set second
				r=1;
			}
			else if(v<t){ // write time to sram since it's better than prev.
				(*(byte*)(SRAM_RANDOMDATA+(10*randLevelDiff)))=(1<<7)|(min&0x3F); // set minute
				(*(byte*)(SRAM_RANDOMDATA+(10*randLevelDiff)+1))=(sec&0x3F); // set second
				r=1;
			}
		}
		// todo: check for GAME_STATUS_FAILED and write time/status to sram
		// if necessary (e.g. player doesn't already have a successful run)
	}
	// infinite random mode stores the number of levels, and for some reason, the time as well.
	else if(gameMode==2){
		if((gameLevel+1)==(randLevelDiff+1)*10){
			// stuff that I don't feel like doing at the moment.
		}
	}

	/* shared parts */
	// time (0x22C8)
	printNum(0x22C8,min,2); // minutes
	printNum(0x22CB,sec,2); // seconds

	// life (0x22D6) (needs fixing for new numLives? seems to work for me)
	if(numLives<7){
		vram_adr(0x22D6+(numLives));
		for(i=0;i<(7-numLives);i++){ vram_put(LIFEMETER_MENU_OFF); }
	}

	// record text (0x228A)
	if(gameStatus!=GAME_STATUS_FAILED){
		if((gameLevel+1)==(randLevelDiff+1)*10 && r==1){
			vram_write((byte*)result_RandomRecord,0x228A,12);
		}
	}

	/* unique parts: */
	// todo: add resultRandom_TopBox text (0x20A8-0x20B7)
	//vram_write((byte*)resultRandom_TopBox[gameStatus==GAME_STATUS_FAILED?1:0],0x20A8,16);

	if(gameStatus==GAME_STATUS_FAILED){
		// header (0x2108): "Rounds Cleared"
		vram_write((byte*)result_RandomCleared,0x2108,15);

		// todo: show game over picture (0x2126)
	}
	else{
		// header (0x2108): "Round Progress"/"Infinite Mode"
		vram_write((byte*)resultRandom_Headers[gameMode-1],0x2108,16);

		// random normal
		if(gameMode==1){
			// inside (begins at 0x2126)
			// todo: breaks past 10 stages; think about a fix after the demo.

			// draw down bigtiles
			for(i=0;i<((randLevelDiff+1)*10);i++){
				i16=0x2126+(i*2);
				vram_adr(i16);
				for(k=0;k<2;k++){ vram_put(k==1?BIGTILE_MENU_DOWN+1:BIGTILE_MENU_DOWN); }
				vram_adr(i16+BIGTILE_HALFROW);
				for(k=0;k<2;k++){ vram_put(k==1?BIGTILE_MENU_DOWN+3:BIGTILE_MENU_DOWN+2); }
			}

			// draw up bigtiles
			for(i=0;i<gameLevel+1;i++){
				i16=0x2126+(i*2);
				vram_adr(i16);
				for(k=0;k<2;k++){ vram_put(k==1?BIGTILE_MENU_UP+1:BIGTILE_MENU_UP); }
				vram_adr(i16+BIGTILE_HALFROW);
				for(k=0;k<2;k++){ vram_put(k==1?BIGTILE_MENU_UP+3:BIGTILE_MENU_UP+2); }
			}
			j16=i16;

			// rebuild tiles from infinite random mode
			vram_adr(0x2271);
			vram_put(0x90);
			vram_put(0x90);
			vram_put(0x9E);

			// footer (0x2274)
			printNum(0x2274,gameLevel+1,2);
			printNum(0x2277,(randLevelDiff+1)*10,2);
		}
		// random infinite
		else if(gameMode==2){
			// inside (begins at 0x2126)

			vram_adr(0x2271);
			vram_put(0x9E);
			// footer (0x2272)
			printNum(0x2272,gameLevel+1,5);
			vram_adr(0x2277);
			vram_put(0x2F); // slash
			vram_put(0x83); // infinite
		}
	}
}

// overall result screen controller
void resultScreen(void){
	cnrom_chrbank(0);
	resetMasterUpdateList();
	oam_clear();
	clearOamTiles();
	pal_bg(palBG_TitleOld);
	pal_spr(palSPR_Sprite1);

	unrle_vram((gameMode==2)?nam_randomComplete:nam_levelComplete,0x2000);

	pad=0;

	// setup screens with the necessary static graphics
	if(gameMode==2){ resultScreen_Random(); }
	else{ resultScreen_Normal(); }

	ppu_on_all();
	pal_fade_to(4,3);

	while(1){
		// todo: add sprites for other game modes
		if(gameMode==2&&gameLevel<9){
			// show cursor on next level to be played
			curSprite=16;
			for(i=0;i<4;++i){
				x=(i%2)?8:0;
				y=(i>1)?8:0;
				curSprite=oam_spr(
					(48+x)+((gameLevel+1)*16), (71+y),
					BIGTILE_CURSOR, chrTest_cAttr[i], curSprite
				);
				curSprite-=8;
			}
		}

		ppu_waitnmi();
		++frameCount;
		rand8();

		pad=pad_trigger(0);
		if(pad&PAD_A || pad&PAD_START){ break; }
	}

	// prepare next screen
	if(gameMode==0){ curScreen=2; } // normal mode, go back to level list
	else{
		if(gameMode==2){ // random set (gameMode==2)
			if(gameLevel==numRandSetLevels){
				// last level has been completed
				curScreen=3; // temp.
			}
			else{
				// another level coming up
				++gameLevel;
				curScreen=4;
				rand8();
			}
		}
		else if(gameMode==1){ // random single (gameMode==1)
			curScreen=3;
			rand8();
		}
	}
	pal_fade_to(0,3);
}

/** Credits Screen ************************************************************/
void credits(void){
	cnrom_chrbank(1);
	resetMasterUpdateList();
	oam_clear();
	clearOamTiles();
	pal_bg(palBG_Credits);
	pal_spr(palSPR_Sprite1);
	unrle_vram(nam_credits,0x2000);

	ppu_on_all();
	pal_fade_to(4,3);

	while(1){
		// draw freemco logo
		oam_meta_spr(96,186,curSprite,ms_freemcoLogo);

		ppu_waitnmi();
		++frameCount;
		rand8();

		pad=pad_trigger(0);
		if(pad!=0){ break; }
	}

	// prepare next screen
	curScreen=1;
	gameMode=0;
	pal_fade_to(0,3);
}

/** Game Screen ***************************************************************/
// Loads a level for Normal Mode.
void game_loadLevel(void){
	// clear puzzle solution so a new one can be loaded.
	memfill(&gameCurrentPuzzleSol,0x00,MAX_INTERNAL_PUZZLE_SIZE);

	s=(puzzleData[gameLevel])[0];
	n=s>>4;		// get difficulty. todo: do something with it.
	s=s&0x0F;	// mask out puzzle size

	for(i=0;i<puzzleSizeBytes[s];i++){
		z = (puzzleData[gameLevel])[1+i];
		gameCurrentPuzzleSol[0+(i*8)] = (z&0x80)>>7;
		gameCurrentPuzzleSol[1+(i*8)] = (z&0x40)>>6;
		gameCurrentPuzzleSol[2+(i*8)] = (z&0x20)>>5;
		gameCurrentPuzzleSol[3+(i*8)] = (z&0x10)>>4;
		gameCurrentPuzzleSol[4+(i*8)] = (z&0x08)>>3;
		gameCurrentPuzzleSol[5+(i*8)] = (z&0x04)>>2;
		gameCurrentPuzzleSol[6+(i*8)] = (z&0x02)>>1;
		gameCurrentPuzzleSol[7+(i*8)] = z&0x01;
	}
	s+=5;

	// set width/height for game
	puzzleW=s; puzzleH=s;
	z=(s*s); // for solution checking
}

// Loads a level for Random Modes.
// todo: make this way more intelligent
void game_loadRandom(void){
	// clear puzzle solution so a new one can be loaded.
	memfill(&gameCurrentPuzzleSol,0x00,MAX_INTERNAL_PUZZLE_SIZE);

	if(!*(byte*)GAMEDEBUG_ACTIVE){
		// Set up parameters for level generation:

		// Random Single Mode
		if(gameMode==1){
			// if not using a custom seed, generate one.
			if(useSeed==0){ levelSeed = rand16(); }
			set_rand(levelSeed);

			// check difficulty (via randLevelDiff)

			// check size and generate puzzle.
			s=randLevelSize+5;
		}
		// Random Set Mode
		else if(gameMode==2){
			// find out where we are in the set (via gameLevel)
			// see what settings this level has
		}

		// set width/height for game
		puzzleW=s; puzzleH=s;
		z=(s*s); // for solution checking
	}

	// prepare randPuzzleBytes
	memfill(&randPuzzleBytes,0x00,MAX_LEVEL_BYTES);

	// generate a random puzzle, mindful of puzzle size.
	s-=5;
	for(i=0;i<(puzzleSizeBytes[s]);i++){
		q = rand8();
		randPuzzleBytes[i]=q;

		gameCurrentPuzzleSol[0+(i*8)] = (q&0x80)>>7;
		gameCurrentPuzzleSol[1+(i*8)] = (q&0x40)>>6;
		gameCurrentPuzzleSol[2+(i*8)] = (q&0x20)>>5;
		gameCurrentPuzzleSol[3+(i*8)] = (q&0x10)>>4;
		gameCurrentPuzzleSol[4+(i*8)] = (q&0x08)>>3;
		gameCurrentPuzzleSol[5+(i*8)] = (q&0x04)>>2;
		gameCurrentPuzzleSol[6+(i*8)] = (q&0x02)>>1;

		// last row handler
		if(i==(puzzleSizeBytes[s])-1){
			switch(s){
				case 0: // 5x5
				case 2: // 7x7
				case 4: // 9x9
				case 6: // 11x11
				case 8: // 13x13
				case 0x0A: // 15x15
					gameCurrentPuzzleSol[7+(i*8)] = (q&0x80);
					randPuzzleBytes[i]=(q&0x80);
					break;
				case 1: // 6x6
				case 5: // 10x10
				case 9: // 14x14
					gameCurrentPuzzleSol[7+(i*8)] = (q&0xF0);
					randPuzzleBytes[i]=(q&0xF0);
					break;
				default: // 8x8, 12x12, 16x16: use all parts
					gameCurrentPuzzleSol[7+(i*8)] = (q&0xFF);
					randPuzzleBytes[i]=(q&0xFF);
					break;
			}
		}
		// normal row handler
		else{ gameCurrentPuzzleSol[7+(i*8)] = q&0x01; }
	}

	// todo: check difficulty here and make adjustments as needed.
}

// throw the solution numbers into vram
// maximum of 7 numbers per row/column. t is for stream counTer.
// todo: sometimes rows/cols won't get drawn. why is this?
void game_drawNumbers(void){
	// row numbers
	vram_inc(0); // +1
	for(i=0;i<puzzleH;i++){
		t=0; v=0; u=0;
		memfill(solDrawArray,0,7); // clear solDrawArray

		for(j=0;j<puzzleW;j++){
			if(j>0){
				//normal handling
				if(gameCurrentPuzzleSol[(i*puzzleH)+j]!=v){
					// the values differ.

					if(gameCurrentPuzzleSol[(i*puzzleH)+j]==0){
						// end a run.
						if(t>0){ solDrawArray[u]=t; t=0; u++; }
					}
					else if(gameCurrentPuzzleSol[(i*puzzleH)+j]==1){
						// begin a new run.
						++t;
					}
				}
				else{
					// the values are the same.
					if(gameCurrentPuzzleSol[(i*puzzleH)+j]==1){ t++; }
				}
			}
			else{
				// special j==0 handling
				if(gameCurrentPuzzleSol[(i*puzzleH)+j]==1){ ++t; }
			}

			v=gameCurrentPuzzleSol[(i*puzzleH)+j]; // store this as previous tile
		}
		// handle any unhandled numbers
		if(t>0){ solDrawArray[u]=t; u++; }

		// bigtile first row:0x2183-0x2189, 0x40 vertical gap
		// onetile first row:0x2163-0x2169, 0x20 vertical gap
		if(puzzleH<=8&&puzzleW<=8){ k16=0x2183+(i*BIGTILE_ROW); }
		else{ k16=0x2163+(i*ONETILE_ROW); }
		vram_adr(k16);
		if(u>0){
			vram_adr(k16+(7-u));
			for(v=0;v<u;v++){ vram_put((solDrawArray[v]-1)+NUMBERS_TYPE1_BEGIN); }
		}
		else{ vram_adr(k16+6); vram_put(NUMBERS_TYPE1_0); }
	}

	// column numbers
	vram_inc(1); // +32
	for(i=0;i<puzzleW;i++){
		t=0; v=0; u=0;
		memfill(solDrawArray,0,7); // clear solDrawArray
		for(j=0;j<puzzleH;j++){
			if(j>0){
				if(gameCurrentPuzzleSol[(j*puzzleH)+i]!=v){
					// values differ.

					if(gameCurrentPuzzleSol[(j*puzzleH)+i]==0){
						// end a run.
						if(t>0){ solDrawArray[u]=t; t=0; u++; }
					}
					else if(gameCurrentPuzzleSol[(j*puzzleH)+i]==1){
						// begin a new run.
						++t;
					}
				}
				else{
					// the values are the same.
					if(gameCurrentPuzzleSol[(j*puzzleH)+i]==1){ t++; }
				}
			}
			else{
				// special j==0 handling
				if(gameCurrentPuzzleSol[(j*puzzleH)+i]==1){ t++; }
			}

			v=gameCurrentPuzzleSol[(j*puzzleH)+i]; // store this as previous tile
		}
		// handle any unhandled numbers
		if(t>0){ solDrawArray[u]=t; u++; }

		// bigtile first col:0x204D-0x210D, 0x02 horiz. gap
		// onetile first col:0x204C-0x210C, 0x01 horiz. gap
		if(puzzleH<=8&&puzzleW<=8){ k16=0x204D+(i*0x02); }
		else{ k16=0x204C+(i*0x01); }
		vram_adr(k16);
		if(u>0){
			vram_adr(k16+((7-u)*0x20));
			for(v=0;v<u;v++){
				vram_put((solDrawArray[v]-1)+NUMBERS_TYPE1_BEGIN);
			}
		}
		else{
			k16=(puzzleW<=8&&puzzleW<=8)?0x210D:0x210C;
			if(puzzleH<=8&&puzzleW<=8){ k16+=(i*0x02); }
			else{ k16+=(i*0x01); }
			vram_adr(k16);
			vram_put(NUMBERS_TYPE1_0);
		}
	}

	vram_inc(0); // reset to +1
}

// throw the puzzle tiles into vram
void game_drawPuzzle(void){
	if(puzzleH<=8&&puzzleW<=8){
		// using bigtiles
		for(i=0;i<puzzleH;i++){
			i16=VRAM_PUZZLE_START+(i*BIGTILE_ROW);
			vram_adr(i16);
			for(j=0;j<puzzleW;j++){
				for(k=0;k<2;k++){ vram_put(k==1?BIGTILE_UP+1:BIGTILE_UP); }
			}
			vram_adr(i16+BIGTILE_HALFROW);
			for(j=0;j<puzzleW;j++){
				for(k=0;k<2;k++){ vram_put(k==1?BIGTILE_UP+3:BIGTILE_UP+2); }
			}
		}

		// oh fuck off (fix the part where bottom two tiles were always down)
		updList_game[35] = BIGTILE_UP+2;
		updList_game[38] = BIGTILE_UP+3;
	}
	else{
		// using onetiles, 1:1 mapping
		for(i=0;i<puzzleH;i++){
			vram_adr(VRAM_PUZZLE_START+(i*ONETILE_ROW));
			for(j=0;j<puzzleW;j++){ vram_put(ONETILE_UP); }
		}
	}
}

// Restarting
void game_restartLevel(void){
	// reset player solution
	memfill(&gamePuzzleData,0x00,MAX_INTERNAL_PUZZLE_SIZE);
	// reset cursor position
	cursorX=0; cursorY=0;

	// we need to restart the level intelligently.

	// in normal mode, this is done with the normal loadLevel and doesn't
	// need a reload. time should be reset to 0:00.
	if(gameMode==0){
		min=0; sec=0;
	}

	// In Random Single and Random Set, we do NOT want to generate a new
	// random level; that would be unfair. The previous puzzle needs to stay
	// in memory. Time is reset in Random Single, but not in Random Set.
	else if(gameMode == 1 || gameMode == 2){
		// random, random infinite. not much of a difference is needed here.

		// reset time in Random Single
		if(gameMode == 1){ min=0; sec=0; }

		// subtract a life for reset if life meter enabled
		if(enableLifeMeter && numLives>0){
			--numLives;

			// update life meter
			updList_game[39]=0x21;
			updList_game[40]=0x03+(numLives);
			updList_game[41]=LIFEMETER_OFF;
		}
	}
}

// compare gamePuzzleData with gameCurrentPuzzleSol to see if all bytes match.
// xxx: this doesn't work for 16x16 puzzles for some reason
void game_checkSolution(void){
	// z is the siZe of the arrays, based on puzzle size. this avoids checking
	// the entire array unless absolutely needed. (e.g. for 16x16 puzzles)
	// there's still a problem with the fact that the entire array is being
	// checked, instead of various other solutions.

	//m=0;
	for(i=0;i<z;i++){
		// try to bail out AS EARLY AS POSSIBLE.
		// (this might be fucking us over on some puzzles)

		// Allow X, ?/dot, and forced X/down tiles to be read properly.
		// if a position is not down, it really doesn't matter what the tile is.

		// old comment (xxx is more recent):
		// allow X/?/dot tiles for non-down positions.
		// xxx: this will cause problems if a forced down tile is used,
		// since it is greater than 1.
		if(gamePuzzleData[i]>1){
			if(gameCurrentPuzzleSol[i]==1) break;
		}
		// we can be more strict.
		else if(gamePuzzleData[i]!=gameCurrentPuzzleSol[i]){
			break;
		}

		// is this the "correct" way to do it?
		if(i==(z-1)){
			// if we reach this, apparently we've won.
			q=i; n=z; // temporary things
			gameStatus = GAME_STATUS_COMPLETE;
		}
	}
}

void game_swapBigTile(void){
	m = gamePuzzleData[(cursorY*puzzleH)+cursorX];

	if(m&4){ return; } // no const tiles
	if(m==2 && xProtection==1){ return; } // no X tile if x protection

	if(enableLifeMeter==1){
		// check solution to see if this tile should not be down.
		if(gameCurrentPuzzleSol[(cursorY*puzzleH)+cursorX]==0 && m==0){
			// update life meter and take a life away.
			--numLives;
			updList_game[39] = 0x21;
			updList_game[40] = 0x03+(numLives);
			updList_game[41] = LIFEMETER_OFF;
			if(numLives==0){ gameStatus=GAME_STATUS_FAILED; }

			// set this tile to forced X and draw it
			gamePuzzleData[(cursorY*puzzleH)+cursorX]=6;
			i16 = VRAM_PUZZLE_START+(cursorY*BIGTILE_ROW)+(cursorX*2);

			updList_game[27] = (i16&0xFF00)>>8;
			updList_game[28] = (i16&0x00FF);
			updList_game[29] = BIGTILE_CONST_X;

			updList_game[30] = (i16&0xFF00)>>8;
			updList_game[31] = (i16&0x00FF)+1;
			updList_game[32] = BIGTILE_CONST_X+1;

			updList_game[33] = ((i16+BIGTILE_HALFROW)&0xFF00)>>8;
			updList_game[34] = ((i16+BIGTILE_HALFROW)&0x00FF);
			updList_game[35] = BIGTILE_CONST_X+2;

			updList_game[36] = ((i16+BIGTILE_HALFROW)&0xFF00)>>8;
			updList_game[37] = ((i16+BIGTILE_HALFROW)&0x00FF)+1;
			updList_game[38] = BIGTILE_CONST_X+3;

			return;
		}
	}

	if(g&&h){
		// holding button AND moving.
		// m is still the current tile in the game puzzle data.
		if(m==k){
			// the old tile is the same type as the new tile.
			gamePuzzleData[(cursorY*puzzleH)+cursorX]=(m==0)?1:0;
			m=gamePuzzleData[(cursorY*puzzleH)+cursorX];
		}
		else{ return; }
	}
	else{
		k=m; // used for holding/moving.
		gamePuzzleData[(cursorY*puzzleH)+cursorX]=(m==0)?1:0;
		m=gamePuzzleData[(cursorY*puzzleH)+cursorX];
	}

	// update bigtile
	i16 = VRAM_PUZZLE_START+(cursorY*BIGTILE_ROW)+(cursorX*2);

	updList_game[27] = (i16&0xFF00)>>8;
	updList_game[28] = (i16&0x00FF);
	updList_game[29] = (m==1)?BIGTILE_DOWN:BIGTILE_UP;

	updList_game[30] = (i16&0xFF00)>>8;
	updList_game[31] = (i16&0x00FF)+1;
	updList_game[32] = (m==1)?BIGTILE_DOWN+1:BIGTILE_UP+1;

	updList_game[33] = ((i16+BIGTILE_HALFROW)&0xFF00)>>8;
	updList_game[34] = ((i16+BIGTILE_HALFROW)&0x00FF);
	updList_game[35] = (m==1)?BIGTILE_DOWN+2:BIGTILE_UP+2;

	updList_game[36] = ((i16+BIGTILE_HALFROW)&0xFF00)>>8;
	updList_game[37] = ((i16+BIGTILE_HALFROW)&0x00FF)+1;
	updList_game[38] = (m==1)?BIGTILE_DOWN+3:BIGTILE_UP+3;
}

// 
void game_swapOneTile(void){
	m = gamePuzzleData[(cursorY*puzzleH)+cursorX];

	if(m&4){ return; }
	if(m==2 && xProtection==1){ return; }

	if(enableLifeMeter==1){
		// check solution to see if this tile should not be down.
		if(gameCurrentPuzzleSol[(cursorY*puzzleH)+cursorX]==0 && m==0){
			// update life meter and take a life away.
			--numLives;
			updList_game[39] = 0x21;
			updList_game[40] = 0x03+(numLives);
			updList_game[41] = LIFEMETER_OFF;
			if(numLives==0){ gameStatus=GAME_STATUS_FAILED; }
			// set this tile to forced X and draw it
			gamePuzzleData[(cursorY*puzzleH)+cursorX]=6;
			i16 = VRAM_PUZZLE_START+(cursorY*BIGTILE_HALFROW)+(cursorX);
			updList_game[27] = (i16&0xFF00)>>8;
			updList_game[28] = (i16&0x00FF);
			updList_game[29] = ONETILE_CONST_X;

			sfx_play(SFX_GAMEERROR,2);
			return;
		}
	}

	if(g&&h){
		// holding button AND moving.
		// m is still the current tile in the game puzzle data.
		if(m==k){
			// the old tile is the same type as the new tile.
			gamePuzzleData[(cursorY*puzzleH)+cursorX]=(m==0)?1:0;
			m=gamePuzzleData[(cursorY*puzzleH)+cursorX];
		}
		else{ return; }
	}
	else{
		k=m; // used for holding/moving.
		gamePuzzleData[(cursorY*puzzleH)+cursorX]=(m==0)?1:0;
		m=gamePuzzleData[(cursorY*puzzleH)+cursorX];
	}

	// update onetile
	i16 = VRAM_PUZZLE_START+(cursorY*BIGTILE_HALFROW)+(cursorX);
	updList_game[27] = (i16&0xFF00)>>8;
	updList_game[28] = (i16&0x00FF);
	updList_game[29] = (m==1)?ONETILE_DOWN:ONETILE_UP;
}

void game_swapTile(void){
	if(puzzleH<=8&&puzzleW<=8){ game_swapBigTile(); }
	else{ game_swapOneTile(); }
}

void game_markBigTile(void){
	// first, check to see if this block is not down.
	m = gamePuzzleData[(cursorY*puzzleH)+cursorX];

	if(m&4){ return; } // don't allow marking of const tiles
	if(m!=0x01){ // not down tile
		if(r&&h){
			// holding+direction
			if(m==k){
				switch(m){
					case 0:
						// up -> x
						gamePuzzleData[(cursorY*puzzleH)+cursorX] = 2;
						sfx_play(SFX_GAMETILEMARK,1);
						break;
					case 2:
						// x -> ?
						gamePuzzleData[(cursorY*puzzleH)+cursorX] = 3;
						sfx_play(SFX_GAMETILEMARK,1);
						break;
					case 3:
						// ? -> up
						gamePuzzleData[(cursorY*puzzleH)+cursorX] = 0;
						sfx_play(SFX_GAMETILEMARK,1);
						break;
					default: break;
				}
			}
		}
		else{
			k=m; // set old tile status
			switch(m){
				case 0:
					// up -> x
					gamePuzzleData[(cursorY*puzzleH)+cursorX] = 2;
					sfx_play(SFX_GAMETILEMARK,1);
					break;
				case 2:
					// x -> ?
					gamePuzzleData[(cursorY*puzzleH)+cursorX] = 3;
					sfx_play(SFX_GAMETILEMARK,1);
					break;
				case 3:
					// ? -> up
					gamePuzzleData[(cursorY*puzzleH)+cursorX] = 0;
					sfx_play(SFX_GAMETILEMARK,1);
					break;
				default: break;
			}
		}

		// finally, draw
		m = gamePuzzleData[(cursorY*puzzleH)+cursorX];
		i16 = VRAM_PUZZLE_START+(cursorY*BIGTILE_ROW)+(cursorX*2);
		// shared update list crap
		updList_game[27] = (i16&0xFF00)>>8;
		updList_game[28] = (i16&0x00FF);
		updList_game[30] = ((i16&0xFF00)>>8);
		updList_game[31] = (i16&0x00FF)+1;
		updList_game[33] = ((i16+BIGTILE_HALFROW)&0xFF00)>>8;
		updList_game[34] = ((i16+BIGTILE_HALFROW)&0x00FF);
		updList_game[36] = ((i16+BIGTILE_HALFROW)&0xFF00)>>8;
		updList_game[37] = ((i16+BIGTILE_HALFROW)&0x00FF)+1;
		// specific tiles
		switch(m){
			case 0:
				updList_game[29] = BIGTILE_UP;
				updList_game[32] = BIGTILE_UP+1;
				updList_game[35] = BIGTILE_UP+2;
				updList_game[38] = BIGTILE_UP+3;
				break;
			case 2:
				updList_game[29] = BIGTILE_X;
				updList_game[32] = BIGTILE_X+1;
				updList_game[35] = BIGTILE_X+2;
				updList_game[38] = BIGTILE_X+3;
				break;
			case 3:
				updList_game[29] = BIGTILE_DOT;
				updList_game[32] = BIGTILE_DOT+1;
				updList_game[35] = BIGTILE_DOT+2;
				updList_game[38] = BIGTILE_DOT+3;
				break;
			default: break;
		}
	}
}

void game_markOneTile(void){
	m = gamePuzzleData[(cursorY*puzzleH)+cursorX];

	if(m&4){ return; } // don't allow marking of const tiles
	if(m!=0x01){ // not down tile
		if(r&&h){
			// holding direction
			if(m==k){
				switch(m){
					// up -> x
					case 0:
						gamePuzzleData[(cursorY*puzzleH)+cursorX] = 2;
						sfx_play(SFX_GAMETILEMARK,1);
						break;
					// x -> ?
					case 2:
						gamePuzzleData[(cursorY*puzzleH)+cursorX] = 3;
						sfx_play(SFX_GAMETILEMARK,1);
						break;
					// ? -> up
					case 3:
						gamePuzzleData[(cursorY*puzzleH)+cursorX] = 0;
						sfx_play(SFX_GAMETILEMARK,1);
						break;
					default: break;
				}
			}
		}
		else{
			k=m; // set old tile status
			switch(m){
				// up -> x
				case 0:
					gamePuzzleData[(cursorY*puzzleH)+cursorX] = 2;
					sfx_play(SFX_GAMETILEMARK,1);
					break;
				// x -> ?
				case 2:
					gamePuzzleData[(cursorY*puzzleH)+cursorX] = 3;
					sfx_play(SFX_GAMETILEMARK,1);
					break;
				// ? -> up
				case 3:
					gamePuzzleData[(cursorY*puzzleH)+cursorX] = 0;
					sfx_play(SFX_GAMETILEMARK,1);
					break;
				default: break;
			}
		}

		m = gamePuzzleData[(cursorY*puzzleH)+cursorX];
		i16 = VRAM_PUZZLE_START+(cursorY*BIGTILE_HALFROW)+(cursorX);
		updList_game[27] = (i16&0xFF00)>>8;
		updList_game[28] = (i16&0x00FF);
		switch(m){
			case 0: g=ONETILE_UP; break;
			case 2: g=ONETILE_X; break;
			case 3: g=ONETILE_DOT; break;
			default: break;
		}
		updList_game[29] = g;
	}
}

void game_markTile(void){
	if(puzzleH<=8&&puzzleW<=8){ game_markBigTile(); }
	else{ game_markOneTile(); }
}

// used for moving between gameplay/pause smoothly
void swapTimeDisplay(void){
	updList_game[0]=pauseStatus?0x24:0x20;
	updList_game[1]=pauseStatus?0xB0:0x84;

	updList_game[3]=pauseStatus?0x24:0x20;
	updList_game[4]=pauseStatus?0xB1:0x85;

	updList_game[6]=pauseStatus?0x24:0x20;
	updList_game[7]=pauseStatus?0xB2:0x86;

	updList_game[9]=pauseStatus?0x24:0x20;
	updList_game[10]=pauseStatus?0xB3:0x87;

	updList_game[12]=pauseStatus?0x24:0x20;
	updList_game[13]=pauseStatus?0xB4:0x88;

	// always show colon in pause menu
	if(pauseStatus) updList_game[8]=0x3A;
}

void game_swapScreens(void){
	resetMasterUpdateList();
	oam_clear();
	clearOamTiles();
	pal_fade_to(1,3);

	pal_bright(0); // secret to masking errors!
	ppu_off();
	cnrom_chrbank(pauseStatus?0:3);

	pal_bg(pauseStatus?
		palBG_Pause :
		stagePalettes[stagePalIndex[stageStyle]+stagePal]
	);

	// clear board on restart.
	if(gameStatus==GAME_STATUS_RESTART){
		ppu_off();
		gameStatus=GAME_STATUS_PLAYING;
		game_drawPuzzle();
		ppu_on_all();
	}

	swapTimeDisplay();
	scroll(pauseStatus?256:0,0);
	ppu_on_all();
	pal_fade_to(4,3);
}

void toggleXProtection(void){
	xProtection=(xProtection==1)?0:1;

	// write new value for X to SRAM
	*(byte*)SRAM_XPROTECT=xProtection;

	// change button graphic
	updList_game[0]=0x26;
	updList_game[1]=0x55;
	updList_game[2]=(xProtection==1)?ONETILE_MENU_UP:ONETILE_MENU_DOWN;

	// toggle "on"/"off" text
	j=0;
	for(i=0;i<2;i++){
		updList_game[3+(i*3)]=0x26;
		updList_game[4+(i*3)]=0x58+i;
		if(xProtection==0){ j='F'; }
		else if(xProtection==1){ j=(i==0)?'N':0x00; }
		updList_game[5+(i*3)]=j;
	}
}

// the in-game pause menu.
void game_pauseMenu(void){
	// update pause menu onetiles
	// previous
	switch(k){
		case 0:	j=0x06; break;
		case 1:	j=0x46; break;
		case 2:	j=0x86; break;
		case 3:	j=0xC6; break;
		default: break;
	}
	updList_game[27]=0x26;
	updList_game[28]=j;
	updList_game[29]=ONETILE_MENU_DOWN;
	// next
	switch(pauseCursor){
		case 0:	j=0x06; break;
		case 1:	j=0x46; break;
		case 2:	j=0x86; break;
		case 3:	j=0xC6; break;
		default: break;
	}
	updList_game[30]=0x26;
	updList_game[31]=j;
	updList_game[32]=ONETILE_MENU_UP;

	if(pad&PAD_UP){
		k=pauseCursor;
		oam_clear();
		if(pauseCursor>0){ --pauseCursor; }
		else{ pauseCursor=3; }
	}
	if(pad&PAD_DOWN){
		k=pauseCursor;
		oam_clear();
		if(pauseCursor<3){ ++pauseCursor; }
		else{ pauseCursor=0; }
	}

	// don't allow players to restart in random modes if it'd cause game over.
	if(gameMode>0 && numLives==1 && pauseCursor==2){
		if(pad&PAD_UP){ pauseCursor=1; }
		else if(pad&PAD_DOWN){ pauseCursor=3; }
	}

	if(pad&PAD_A){
		// unpause
		if(pauseCursor==0){ pauseStatus=0; game_swapScreens(); }

		// toggle x protection
		else if(pauseCursor==1){ toggleXProtection(); }

		// restart
		else if(pauseCursor==2){
			// todo: in random mode, display a popup.
			gameStatus=GAME_STATUS_RESTART;
			pauseStatus=0;

			// fuck the pause menu
			updList_game[30]=0x26;
			updList_game[31]=0x86;
			updList_game[32]=(gameMode!=0 && numLives==1)?ONETILE_MENU_X:ONETILE_MENU_DOWN;

			game_restartLevel();
			delay(1);
			game_swapScreens();
		}
		else if(pauseCursor==3){ // quit
			// clear current puzzle solution
			memfill(&gameCurrentPuzzleSol,0x00,MAX_INTERNAL_PUZZLE_SIZE);
			curScreen=(gameMode>0)?3:2;
			gameStatus=GAME_STATUS_QUIT;
			pal_fade_to(0,3);
		}
	}
}

/* Actual Game Part */
void game_loop(void){
	cnrom_chrbank(3);
	resetMasterUpdateList();
	oam_clear();
	clearOamTiles();

	unrle_vram(gameStyles[stageStyle],0x2000);
	unrle_vram(nam_gamePaused,0x2400); // pause screen in second nametable
	pal_bg(stagePalettes[stagePalIndex[stageStyle]+stagePal]);
	pal_spr(palSPR_Sprite1);

	pad=0;
	n=0;	// debug stuff; temp var otherwise.
	q=0;	// debug stuff; temp var otherwise.
	g=0;	// holding button
	h=0;	// holding direction
	j=0;
	k=0;
	m=0;
	p=0;	// used in puzzle generation (current Place [in row])
	r=0;
	t=0;	// used in row/column number drawing (stream counTer)
	u=0;	// used to keep track of solDrawArray inde). (row/column numbers)
	v=0;	// initally used in row/column number drawing (temp var)
	w=0;	// used in puzzle generation (current roW)
	//
	repeatDir=0;

	gameStatus=GAME_STATUS_PLAYING;
	pauseStatus=0;
	pauseCursor=0;
	gameTick=0;
	cursorX=0; cursorY=0;
	j16=0; // used in puzzle generation. sizes>8 needs 16 bits for one row.

	// level loading/generation
	if(*(byte*)GAMEDEBUG_ACTIVE){ game_loadRandom(); }
	else{
		if(gameMode==0){
			game_loadLevel();
			// increment play count for this level (technically also uses the
			// lower two bits of byte 2, but too lazy to code that right now.)
			r=(*(byte*)(SRAM_NORMALDATA+(4*gameLevel+2)));
			++r;
			(*(byte*)(SRAM_NORMALDATA+(4*gameLevel+2)))=r;
		}
		else{ game_loadRandom(); }
	}

	// clear (fat) internal player puzzle data
	memfill(&gamePuzzleData,0x00,MAX_INTERNAL_PUZZLE_SIZE);

	// draw the puzzle tiles.
	game_drawPuzzle();

	// draw row/column numbers (tiles: 0xB0-0xBF 1-16; 0xC0=0)
	game_drawNumbers();

	// set up update list
	memcpy(updList_game,updListData_game,sizeof(updListData_game));
	set_vram_update(GAME_UPDATE_LIST_COUNT,updList_game);

	// add static puzzle size numbers
	printNum(0x20C8,puzzleW,2);
	printNum(0x20E8,puzzleH,2);

	// Life meter stuff
	if(enableLifeMeter==1){
		// life initialization (xxx: is this condition correct?)
		if(gameMode!=2 || (gameMode==2 && gameLevel==0)){
			numLives=8; // initialize life
		}

		// draw life meter
		vram_adr(0x2103);
		for(i=0;i<numLives;i++){ vram_put(LIFEMETER_ON); }
		// draw empty blocks
		if(numLives<8){
			for(i=0;i<8-numLives;i++){ vram_put(LIFEMETER_OFF); }
		}

		// draw life label
		vram_adr(0x2124);
		for(i=0;i<6;i++){ vram_put(gameLifeLabel[i]); }
	}

	// pause menu stuff //

	// mode text
	vram_write((unsigned char *)gameMode_Pause[gameMode],0x2509,6);

	// puzzle size
	printNum(0x2512,puzzleW,2);
	vram_put(0x89);
	printNum(0x2515,puzzleH,2);

	// level information
	if(gameMode==0){ // normal: "Level #\n[title 16chars]"
		vram_write((byte*)pause_LevelText,0x2548,5);
		printNum(0x254E,(gameLevel+1),3);

		// level name (0x2568)
		vram_adr(0x2568);

		g=(*(byte*)(SRAM_NORMALDATA+(4*gameLevel))&0xC0)>>6;
		if(g==2){
			// show level name
			r=0;
			for(i=0;i<16;i++){
				j=(puzzleNames[gameLevel])[i];
				// check where we are.
				if(j==0xFF){ r=i; }
				if(r!=0){ vram_put(0); }
				else{ vram_put(j); }
			}
			r=0;
		}
		else{
			// show question marks
			for(i=0;i<16;i++){ vram_put(0x3F); }
		}
	}
	else if(gameMode==1){ // random single: ??
		v=0;
		switch(randLevelDiff){
			case 0: case 2: v=4; break;
			default: v=6; break;
		}
		k16=0x2548+v+1;
		vram_write((byte*)pause_DiffNames[randLevelDiff],0x2548,v);
		printNum(k16,(gameLevel+1),2);
	}
	else if(gameMode==2){ // random set: puzzle#/($LIMIT)
		printNum(0x2548,(gameLevel+1),5);
		vram_put(0x2F); // slash
		vram_put(0x83); // inf.
	}

	// x protection box
	vram_adr(0x2655);
	vram_put((xProtection==1)?ONETILE_MENU_UP:ONETILE_MENU_DOWN);
	// initial "n /ff" for x protection on/off
	vram_adr(0x2658);
	vram_put(xProtection==1?'N':'F');
	vram_put(xProtection==1?0:'F');
	// end pause menu stuff

	ppu_on_all();
	frameCount=0;

	// for sanity; check if this is actually needed
	h=0;
	z=puzzleW*puzzleH;

	// always reset time in normal mode. only reset time on the first level
	// in random set, when a new session begins.
	if(gameMode==0 || (gameMode!=0 && gameLevel==0)){
		min=0; sec=0;
	}

	q=0;

	pal_fade_to(4,3);
	while(1)
	{
		if(gameStatus==GAME_STATUS_COMPLETE || gameStatus==GAME_STATUS_FAILED
			|| gameStatus==GAME_STATUS_QUIT){
			break;
			// how each status reacts is dealt with below.
		}

		// sprite time
		if(!pauseStatus){
			oam_clear();
			if(puzzleW>8){
				if(frameCount%8==0){
					++q;
					if(q>3){ q=q%4; }
				}

				// using 8px smalltiles; add 8px onetile cursor
				curSprite=1<<3;
				curSprite=oam_spr(
					96+(cursorX<<3), 87+(cursorY<<3),
					smallCursorFrames[q], 2, curSprite
				);
				// top arrow
				curSprite=oam_spr(96+(cursorX<<3),72,(blinky)?SPR_ARROW_DOWN+0x10:SPR_ARROW_DOWN,3,curSprite);
				// left arrow
				curSprite=oam_spr(82,87+(cursorY<<3),(blinky)?SPR_ARROW_RIGHT+0x10:SPR_ARROW_RIGHT,3,curSprite);
			}
			else{
				// using 16px bigtiles; add 16px bigtile cursor
				if(frameCount%8==0){
					++q;
					if(q>3){ q=q%4; }
				}

				curSprite=1<<6;
				for(i=0;i<4;++i)
				{
					x=(i%2)?8:0;
					y=(i>1)?8:0;
					curSprite=oam_spr(
						(96+x)+(cursorX<<4), (87+y)+(cursorY<<4),
						bigCursorFrames[q], chrTest_cAttr[i], curSprite
					);
					curSprite-=8;
				}
				// top arrow
				for(i=0;i<2;i++){
					curSprite=oam_spr((96+(cursorX<<4)+(i*8)),72,(blinky)?(SPR_BIGARROW_DOWN+0x10)+i:SPR_BIGARROW_DOWN+i,3,curSprite);
					curSprite-=8;
				}
				// left arrow
				for(i=0;i<2;i++){
					curSprite=oam_spr(82,87+(cursorY<<4)+(i*8),(blinky)?(SPR_BIGARROW_LEFT+0x10)+i:SPR_BIGARROW_LEFT+i,3,curSprite);
					curSprite-=8;
				}
			}
		}
		else{
			curSprite=1<<3;

			m=(pauseCursor<<4);

			curSprite=oam_spr(48,127+m,ONETILE_CURSOR,3,curSprite);
			curSprite-=16;

			// todo: figure out how the cursor should map to the puzzle size
			// and internal solution array.
			// one row of the solution part is 32 tiles.
			n=(cursorY*puzzleH)+cursorX;

			if(n<32){ y=0; }
			else if(n>31 && n<64){ y=1; }
			else if(n>63 && n<96){ y=2; }
			else if(n>95 && n<128){ y=3; }
			else if(n>127 && n<160){ y=4; }
			else if(n>159 && n<192){ y=5; }
			else if(n>191 && n<224){ y=6; }
			else if(n>223){ y=7; }

			x=0; y=0;
		}

		ppu_waitnmi();
		++frameCount;
		if(!pauseStatus) ++gameTick;
		rand8();

		// pad input (poll first, then check pad_state)
		if(pauseStatus){ pad=pad_trigger(0); }
		else{ pad=pad_poll(0); }

		if(pad&PAD_START){
			p=pauseStatus;
			pauseStatus^=1;
			// reset up tile if not on the first option
			if(p==1 && pauseCursor!=0){
				switch(pauseCursor){
					case 1:	j=0x46; break;
					case 2:	j=0x86; break;
					case 3:	j=0xC6; break;
					default: break;
				}
				updList_game[30]=0x26;
				updList_game[31]=j;
				updList_game[32]=ONETILE_MENU_DOWN;
			}
			pauseCursor=0;
			game_swapScreens();
		}

		// handle pause menu
		if(pauseStatus){
			game_pauseMenu();
			continue;
		}

		// h is for Holding (direction); g is for holdinG (button).
		pad=pad_state(0);

		if(!pad){
			if(h){ h=0; repeatDir=0; }
			if(g){ g=0; } // A
			if(r){ r=0; } // B
		}
		// direction hold
		if(repeatDir>0){ pad=0; --repeatDir; }
		// check if A/B is being pressed and react accordingly.
		if(!pad&PAD_A && !g){ g=0; }
		if(!pad&PAD_B && !r){ r=0; }

		// movement
		if(pad&PAD_UP){
			if(!h){ repeatDir=16; }else{ repeatDir=3; }
			h=1;
			if(cursorY>0){
				--cursorY;
				sfx_play(SFX_GAMEMOVE,0);
			}
		}
		if(pad&PAD_DOWN){
			if(!h){ repeatDir=16; }else{ repeatDir=3; }
			h=1;
			if(cursorY<(puzzleH-1)){
				++cursorY;
				sfx_play(SFX_GAMEMOVE,0);
			}
		}
		if(pad&PAD_LEFT){
			if(!h){ repeatDir=16; }else{ repeatDir=3; }
			h=1;
			if(cursorX>0){
				--cursorX;
				sfx_play(SFX_GAMEMOVE,0);
			}
		}
		if(pad&PAD_RIGHT){
			if(!h){ repeatDir=16; }else{ repeatDir=3; }
			h=1;
			if(cursorX<(puzzleW-1)){
				++cursorX;
				sfx_play(SFX_GAMEMOVE,0);
			}
		}

		// buttons
		// A button: swap block at current cursor position
		if(pad&PAD_A){
			if(!g){
				r=0;
				g=1;
				game_swapTile();
			}
			else if(g&&h){
				game_swapTile();
			}
			game_checkSolution();
		}
		// B button: mark block/change mark at current position
		if(pad&PAD_B){
			if(!r){
				r=1;
				g=0;
				game_markTile();
			}
			else if(r&&h){
				game_markTile();
			}
		}

		// clockblockin'
		blinky = (gameTick&32)?1:0;
		if(gameTick%64==0) ++sec;
		if(sec == 60){ ++min; sec=0; }

		// stop clock (and gameplay) at 60:00
		// todo: disable this limit in random set?
		if(min<60){
			updList_game[2]=(min>9)?0x30+((min/10)%10):0;
			updList_game[5]=0x30+(min%10);
			updList_game[11]=0x30+(sec/10%10);
			updList_game[14]=0x30+(sec%10);
		}
		else{
			gameStatus=GAME_STATUS_FAILED;
			updList_game[2]=0x30+6;
			updList_game[5]=0x30;
			updList_game[11]=0x30;
			updList_game[14]=0x30;
		}
		// always update colon
		updList_game[8]=(blinky)?0x6E:0x3A;

		// player position
		updList_game[17]=0x30+(((cursorX+1)/10)%10);
		updList_game[20]=0x30+((cursorX+1)%10);
		updList_game[23]=0x30+(((cursorY+1)/10)%10);
		updList_game[26]=0x30+((cursorY+1)%10);
	}

	// post-loop zone //
	if(gameStatus==GAME_STATUS_COMPLETE){
		// puzzle was solved; go to the results screen.
		// todo: show "clear!" sprite
		wait=32;
		while(wait){
			ppu_waitnmi();
			rand8();
			--wait;
		}
		curScreen=5;
	}
	else if(gameStatus==GAME_STATUS_FAILED){
		// failed attempt. as for how it's done, that depends on the game mode.
		// todo: show "failed" sprite
		wait=32;
		while(wait){
			ppu_waitnmi();
			rand8();
			--wait;
		}

		// normal mode: failed via time limit (no other way to fail...)
		if(gameMode==0){
			// todo: set failed status in sram.
			curScreen=2; // go back to normal mode menu.
		}
		// random mode: most likely failed from life depleting
		else{
			// only deal with the status part in non-infinite random.
			if(gameMode==1){
				// check status of current difficulty
				s=(*(byte*)(SRAM_RANDOMDATA+(10*randLevelDiff))&0x80)>>7;
				if(s!=1){ // don't do this if we've already passed before.
					// todo: write time, which is a pain.
					//*(byte*)(SRAM_RANDOMDATA+(10*randLevelDiff))=0<<7;
				}
			}

			curScreen=3; // go back to random mode menu.
		}
	}
	else if(gameStatus==GAME_STATUS_QUIT){
		// quit game, called from pause menu (in second nametable)
		scroll(0,0);
	}

	// debug cockblock
	/*
	if(*(byte*)GAMEDEBUG_ACTIVE==1){
		s=puzzleH; // reset puzzle size for debug menu
		curScreen=0xD8; // back to ecp debug menu
	}
	*/

	// prepare next screen
	//reset_vram(1);
	pal_fade_to(0,3);
	// clear second nametable
	vram_adr(0x2400);
	vram_fill(0,0x3C0);
}

/** kill screen kill **********************************************************/
void killScreen(){
	cnrom_chrbank(0);
	resetMasterUpdateList();
	pal_bg(palBG_MainMenu);
	pal_spr(palSPR_Sprite1);
	unrle_vram(nam_killscreen,0x2000);

	if(*(byte*)HELLO == 0x0C){
		// GAYSEX
		// todo: scramble this message: "Entered password: GAYSEX"
	}
	else{
		// crashed in some other fashion
	}

	//set_vram_update(MAX_MASTER_UPDATE_LIST_COUNT,masterUpdList);

	pal_bright(4);
	ppu_on_all();
	pal_bright(4);

	while(1){
		ppu_waitnmi();
	}
}

/** and everything else. ******************************************************/

void initVars(void){
	g=0; h=0; i=0; j=0;
	k=0; m=0; n=0; p=0; // <K> mno pees
	q=0; r=0; s=0; t=0;
	u=0; v=0; w=0; x=0;
	y=0; z=0;
	i16=0; j16=0;
	k16=0; m16=0;

	//masterUpdList_Size = 0;

	curScreen=0;
	gameTick=0;
	gameMode=0;			// 0x00=normal,0x01=random single,0x02=random set
	gameLevel=0;		// numerical game level
	numLives=0;			// life meter
	repeatDir=0;
	enableLifeMeter=0;	// 0x00=disabled,0x01=enabled

	// normal mode
	pageNum=0;			// normal mode page number

	// random mode
	randLevelSize=0;	//  
	randLevelDiff=0;	// 0x00=easy,0x01=medium,0x02=hard,0x03=expert,0x04=master
	useSeed=0;			// random single
	levelSeed=0;
	numRandSetLevels=0;

	pauseStatus=0;
	pauseCursor=0;

	// gameplay style craps; this is kind of hardcoded on purpose for now.
	stageStyle=0;
	stagePal=0;

	blinky=0;
	curSprite=0;
	randWord=0;

	explainCount=0;
	explainLine=0;

	bright=4;

	// sram temp.
	// xxx: is accessing this before checking sram consistency a good idea?
	xProtection = *(byte*)SRAM_XPROTECT;

	// reset debug flags
	*(byte*)GAMEDEBUG_ACTIVE=0;	// required reset
#if _DEBUGMODE_
	*(byte*)DEBUGMODE_ACTIVE=1;	// activate
#else
	*(byte*)DEBUGMODE_ACTIVE=0;	// deactivate
#endif // _DEBUGMODE_
}

void main(void)
{
	// initialization stuffs
	bank_bg(0);
	bank_spr(1);
	oam_clear();
	clearOamTiles();

	// clear nametables in case of a reset.
	vram_adr(0x2000);
	vram_fill(0,0x3C0);
	vram_adr(0x2400);
	vram_fill(0,0x3C0);

	// MYSTERY CHECK!! WHO COULD EVER TRIGGER THIS???
	if(*(byte*)HELLO == 0x0C){ killScreen(); return; }

	initVars();
	if(sramCheckHeader()!=0){
		// kill sram
		memfill((void*)0x6008,0x00,0x1FF8);
	}

	// check reset signature. if warm boot, go to title.
	// otherwise, show prerelease screen.
	curScreen=(checkResetSignature()==1)?0:0xD0;

	addResetSignature(); // "freem" @ 0x0720
	// todo: if PAL, put "amiga rules" @ 0x0730

	// play with the random number generator
	while(k<10){
		rand8();
		if(rand8()%2==0) ++k;
	}
	k=0;

	// main loop
	while(1)
	{
		switch(curScreen)
		{
			case 0x00: titleScreen(); break;
			case 0x01: mainMenu(); break;
			case 0x02: normalMenu(); break;
			case 0x03: randomMenu(); break;
			case 0x04: game_loop(); break;
			case 0x05: resultScreen(); break;
			case 0xD0: prerelScreen(); break;	// temp; remove for final
			case 0xFF: credits(); break;		// temp value?
			default: killScreen(); break;		// KILL SCREEN KILL
		}
	}

	// if we get down here, something REALLY went wrong and we should display
	// information in the Magic Incantations section.
	killScreen();
}
