/* picross.h - super shared family picross */
#ifndef PICROSS_H
#define PICROSS_H

// memory location defines
#define RESET_COUNTER		0x0707
#define RESET_FREEMSIG		0x0720
#define RESET_AMIGASIG		0x0730
// used with the sabu code
#define GAMEDEBUG_ACTIVE	0x0747
// freem debugging and such
#define DEBUGMODE_ACTIVE	0x0748

// tile defines
#define LIFEMETER_ON		0x0D
#define LIFEMETER_OFF		0x0E
#define LIFEMETER_MENU_ON	0xF3
#define LIFEMETER_MENU_OFF	0xF4

#define BIGTILE_UP			0x90
#define BIGTILE_DOWN		0x94
#define BIGTILE_DOT			0x98
#define BIGTILE_X			0x9C
#define BIGTILE_CONST_DOWN	0xA0
#define BIGTILE_CONST_X		0xA4

#define ONETILE_UP			0xA8
#define ONETILE_DOWN		0xA9
#define ONETILE_DOT			0xAA
#define ONETILE_X			0xAB
#define ONETILE_CONST_DOWN	0xAC
#define ONETILE_CONST_X		0xAD

#define BIGTILE_MENU_UP				0xB0
#define BIGTILE_MENU_DOWN			0xB4
#define BIGTILE_MENU_DOT			0xB8
#define BIGTILE_MENU_X				0xBC
#define BIGTILE_MENU_CONST_DOWN		0xC0
#define BIGTILE_MENU_CONST_X		0xC4

#define ONETILE_MENU_UP				0xC8
#define ONETILE_MENU_DOWN			0xC9
#define ONETILE_MENU_DOT			0xCA
#define ONETILE_MENU_X				0xCB
#define ONETILE_MENU_CONST_DOWN		0xCC
#define ONETILE_MENU_CONST_X		0xCD

// puzzle solution numbers (type1 = transparent BG; type2 = solid BG)
#define NUMBERS_TYPE1_BEGIN			0xB0
#define NUMBERS_TYPE1_0				0xC0
#define NUMBERS_TYPE2_BEGIN			0xD0
#define NUMBERS_TYPE2_0				0xE0

// sprite defines
#define BIGTILE_CURSOR		0x0A
#define ONETILE_CURSOR		0x0B
#define SPR_ARROW_RIGHT		0x0C
#define SPR_ARROW_DOWN		0x0D
#define SPR_ARROW_LEFT		0x0E
#define SPR_ARROW_UP		0x0F
#define SPR_BIGARROW_LEFT	0x04
#define SPR_BIGARROW_DOWN	0x06
#define MENU_POD			0x08
#define MENU_DOT			0x09

// game defines
#define GAME_MODE_NORMAL		0x00
#define GAME_MODE_RANDOM		0x01
#define GAME_MODE_INFRANDOM		0x02

#define GAME_STATUS_PLAYING		0x00
#define GAME_STATUS_RESTART		0x01
#define GAME_STATUS_COMPLETE	0x02
#define GAME_STATUS_FAILED		0x03
#define GAME_STATUS_QUIT		0x04

#define ONETILE_ROW		0x20
#define BIGTILE_HALFROW	0x20
#define BIGTILE_ROW		0x40

#define VRAM_PUZZLE_START		0x216C

// master update list.
#define MAX_MASTER_UPDATE_LIST_COUNT 32
byte masterUpdList[MAX_MASTER_UPDATE_LIST_COUNT*3];
//byte masterUpdList_Size; // xxx: do I really need this?

// Sound Effects
#define SFX_START				0		// Start on Title, Select on Main Menu
#define SFX_MAINMENUCHANGE		1		// Main Menu; Normal Menu Life Meter toggle
#define SFX_MENUCHANGEDOWN		2		// used in other menus
#define SFX_MENUCHANGEUP		3		// used in other menus
#define SFX_MENUCHANGEOPTION	4		// used in Random Menu
#define SFX_MAINMENUSELECT		5		// Only used on Main Menu
#define SFX_MENUBACK			6		// used when backing out of a menu
#define SFX_GAMEMOVE			7		// used when moving
#define SFX_GAMETILEDOWN		8		// press A to mark tile down.
#define SFX_GAMETILEMARK		9		// press B to switch tile mark.
#define SFX_GAMEERROR			10		// used when uncovering constant X

// SRAM stuff
#define SRAM_HEADER			0x6000
#define SRAM_VERSION		(SRAM_HEADER+4)
#define SRAM_CHKSUMLOC		(SRAM_HEADER+5)
#define SRAM_XPROTECT		(SRAM_HEADER+7)
#define SRAM_NORMALDATA		(SRAM_HEADER+8)			// 4 bytes per entry
#define SRAM_RANDOMDATA		(SRAM_NORMALDATA+768)	// 2 bytes per entry
#define SRAM_INFRANDOMDATA	(SRAM_RANDOMDATA+10)	// 4 bytes

// other stuff
#define HELLO					0xDA78				// game genie code "GAYSEX" modifies this byte
#define ILOVEYOU				0xBF10
#define WONTYOUTELLMEYOURNAME	0x8EED

// screen nametables //
#include "_nam/nam_title.h"

// menus
#include "_nam/nam_menuMain.h"
#include "_nam/nam_menuNormal3.h"
#include "_nam/nam_RandomSingle.h"
#include "_nam/nam_RandomSet.h"

// gameplay
#include "_nam/nam_gameClassic.h"
#include "_nam/nam_gameChecker.h"
#include "_nam/nam_gamePaused.h"

// results
#include "_nam/nam_levelComplete.h"
#include "_nam/nam_randomComplete.h" // xxx: non-infinite by default

// other screens
#include "_nam/nam_credits.h"
#include "_nam/nam_killscreen.h"
// temp; remove when needed
#include "_nam/nam_prerelease.h"

// palettes
#include "palettes.h"

/* metasprites */
// freemco logo sprite (chr0,1; 8x2)
const byte ms_freemcoLogo[]={
	0,0,0xf0,2, 8,0,0xf1,2, 16,0,0xf2,2, 24,0,0xf3,2, 32,0,0xf4,2, 40,0,0xf5,2, 48,0,0xf6,2, 56,0,0xf7,2,
	0,8,0xf8,2, 8,8,0xf9,2, 16,8,0xfa,2, 24,8,0xfb,2, 32,8,0xfc,2, 40,8,0xfd,2, 48,8,0xfe,2, 56,8,0xff,2,
	128
};

// gameplay "CLEAR!" sprite (chr3; 6x2)
const byte ms_Clear[]={
	-24,-8,0x30,0,	-16,-8,0x31,0,	-8,-8,0x32,0,	0,-8,0x33,0,	8,-8,0x34,0,	16,-8,0x35,0,
	16,0,0x45,0,	8,0,0x44,0,		0,0,0x43,0,		-8,0,0x42,0,	-16,0,0x41,0,	-24,0,0x40,0,
	128
};

// gameplay "FAILED" sprite (chr3; 6x2)
const unsigned char ms_Failed[]={
	-24,-8,0x36,0,	-16,-8,0x33,0,	-8,-8,0x37,0,	0,-8,0x31,0,	8,-8,0x32,0,	16,-8,0x38,0,
	16,0,0x48,0,	8,0,0x42,0,		0,0,0x41,0,		-8,0,0x47,0,	-16,0,0x43,0,	-24,0,0x46,0,
	128
};

// bigtile cursor attributes
// 0x40 = hflip; 0x80 = vflip; 0xC0 = double flip
const byte chrTest_cAttr[4]={ 2,0x42,0x82,0xC2 };

// 16px (bigtile) cursor animation tiles
const byte bigCursorFrames[4] = { BIGTILE_CURSOR+0x20, BIGTILE_CURSOR, BIGTILE_CURSOR+0x10, BIGTILE_CURSOR };
// 8px (onetile) cursor animation tiles
const byte smallCursorFrames[4] = { ONETILE_CURSOR, ONETILE_CURSOR+0x10, ONETILE_CURSOR+0x20, ONETILE_CURSOR+0x30 };

// bigtile arrow cursor animation tiles
const byte bigArrowLeft1Frames[4] = { SPR_BIGARROW_LEFT, SPR_BIGARROW_LEFT+0x10, SPR_BIGARROW_LEFT+0x20, SPR_BIGARROW_LEFT+0x10 };
const byte bigArrowLeft2Frames[4] = { (SPR_BIGARROW_LEFT+1), (SPR_BIGARROW_LEFT+1)+0x10, (SPR_BIGARROW_LEFT+1)+0x20, (SPR_BIGARROW_LEFT+1)+0x10 };
const byte bigArrowDown1Frames[4] = { SPR_BIGARROW_DOWN, SPR_BIGARROW_DOWN+0x10, SPR_BIGARROW_DOWN+0x20, SPR_BIGARROW_DOWN+0x10 };
const byte bigArrowDown2Frames[4] = { (SPR_BIGARROW_DOWN+1), (SPR_BIGARROW_DOWN+1)+0x10, (SPR_BIGARROW_DOWN+1)+0x20, (SPR_BIGARROW_DOWN+1)+0x10 };
// onetile arrow cursor animation tiles

// gameplay screen styles
const byte* const gameStyles[2]={
	nam_gameClassic,nam_gameChecker
};

// pre-release screen "press start to continue"
const byte prerel_PressStart[24]="Press START to continue.";

// main menu header text
#define MAINMENU_HEADER_LENGTH 6
// 8x16 tiles version:
const byte menuMain_NormalTop[MAINMENU_HEADER_LENGTH]={0xEE,0xE6,0xE4,0xE5,0xE2,0xEA};
const byte menuMain_NormalBot[MAINMENU_HEADER_LENGTH]={0xFE,0xF6,0xFE,0xF5,0xFE,0xFB};
const byte menuMain_RandomTop[MAINMENU_HEADER_LENGTH]={0xE4,0xE2,0xEE,0xE7,0xE6,0xE5};
const byte menuMain_RandomBot[MAINMENU_HEADER_LENGTH]={0xFE,0xFE,0xFE,0xF7,0xF6,0xF5};
const byte* const menuMain_HeaderTops[2]={ menuMain_NormalTop,menuMain_RandomTop };
const byte* const menuMain_HeaderBots[2]={ menuMain_NormalBot,menuMain_RandomBot };

// main menu explain text
#define MAINMENU_EXPLAIN_LENGTH 110
#define MAINMENU_EXPLAIN_LINELEN 22
const byte menuMain_ExplainNormal[MAINMENU_EXPLAIN_LENGTH]={
	"Challenge a selection " \
	"of puzzles with a 60  " \
	"minute time limit per " \
	"level. Difficulties   " \
	"from Easy to Master.  "
};
const byte menuMain_ExplainRandom[MAINMENU_EXPLAIN_LENGTH]={
	"Play a single random  " \
	"level or a set of     " \
	"random levels. Size,  " \
	"length, and difficulty" \
	"are selectable.       "
};

const byte* const menuMain_ExplainTexts[2]={ menuMain_ExplainNormal,menuMain_ExplainRandom };

// nametable location 0x2369
const byte menuNormal_LifeMeter[14]={"Life Meter ON!"};

// nametable location 0x2124-0x2129
const byte gameLifeLabel[6]={"\x7CLIFE\x7B"};

// Normal Menu difficulty names
const byte menuNormal_DiffEasy[6]={" Easy "};
const byte menuNormal_DiffHard[6]={" Hard "};

// Random Menu difficulty names
const byte menuRandom_DiffEasy[8]={"  Easy  "};
const byte menuRandom_DiffMedium[8]={" Medium "};
const byte menuRandom_DiffHard[8]={"  Hard  "};
const byte menuRandom_DiffExpert[8]={" Expert "};
const byte menuRandom_DiffMaster[8]={" Master "};
const byte menuRandom_DiffInfinite[8]={"Infinite"};

// Game Screen Random Difficulty Names
const byte game_DiffEasy[4]={"EASY"};
const byte game_DiffMedium[4]={"MED."};
const byte game_DiffHard[4]={"HARD"};
const byte game_DiffExpert[5]={"EXPR."};
const byte game_DiffMaster[5]={"MAST."};

// Pause Menu mode names
const byte gameMode_Normal[6]={"Normal"};
const byte gameMode_Random[6]={"Random"};
const byte gameMode_Infinite[6]={"\x83Rand."};
const byte pause_Debug[6]={"Debug!"};

// Pause Menu game names
const byte pause_DiffEasy[4]={"Easy"};
const byte pause_DiffMedium[6]={"Medium"};
const byte pause_DiffHard[4]={"Hard"};
const byte pause_DiffExpert[6]={"Expert"};
const byte pause_DiffMaster[6]={"Master"};

// pointer fun
const byte* const menuNormal_Difficulty[6]={ menuNormal_DiffEasy,pause_DiffMedium,menuNormal_DiffHard,pause_DiffExpert,pause_DiffMaster};
const byte* const menuRandom_Difficulty[6]={menuRandom_DiffEasy,menuRandom_DiffMedium,menuRandom_DiffHard,menuRandom_DiffExpert,menuRandom_DiffMaster,menuRandom_DiffInfinite};
const byte* const game_DiffNames[5]={game_DiffEasy,game_DiffMedium,game_DiffHard,game_DiffExpert,game_DiffMaster};
const byte* const gameMode_Pause[3]={gameMode_Normal,gameMode_Random,gameMode_Infinite};
const byte* const pause_DiffNames[5]={pause_DiffEasy,pause_DiffMedium,pause_DiffHard,pause_DiffExpert,pause_DiffMaster};

const byte pause_LevelText[5]={"Level"};

// Pause Menu dialogs (RandomRestart may end up being the only one used)

const byte pauseDialog_RandomRestart[20*7]={
	0x70,0x1c,0x1c,0x1c,0x1c,0x1c,0x1c,0x1c,0x1c,0x1c,0x1c,0x1c,0x1c,0x1c,0x1c,0x1c,0x1c,0x1c,0x1c,0x71,
	0x1d,0x00,0x00,0x31,0x24,0x32,0x33,0x20,0x31,0x33,0x28,0x2d,0x26,0x00,0x28,0x2d,0x00,0x00,0x00,0x1d,
	0x1d,0x31,0x20,0x2d,0x23,0x2e,0x2c,0x00,0x2c,0x2e,0x23,0x24,0x00,0x22,0x2e,0x32,0x33,0x32,0x00,0x1d,
	0x1d,0x00,0x00,0x2e,0x2d,0x24,0x00,0x2b,0x28,0x25,0x24,0x00,0x34,0x2d,0x28,0x33,0x0e,0x00,0x00,0x1d,
	0x1d,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x1d,
	0x1d,0x00,0x31,0x24,0x20,0x2b,0x2b,0x38,0x00,0x00,0x31,0x24,0x32,0x33,0x20,0x31,0x33,0x3c,0x00,0x1d,
	0x72,0x1c,0x1c,0x67,0x00,0x38,0x24,0x32,0x00,0xcc,0x1c,0x67,0x00,0x2d,0x2e,0x00,0xcc,0x1c,0x1c,0x73
};

// Result screen strings
// * normal mode
//  record text written to to $2330
const byte result_NewRecord[11]={ "New Record!" };

// * random mode
// top box header
const byte result_RandomComplete[16]={ "Puzzle Complete!" };
const byte result_RandomNoLives[16]={ " No More Lives! " };
const byte* const resultRandom_TopBox[2]={result_RandomComplete,result_RandomNoLives};

// inner box header
const byte result_RandomNormal[16]={ " Round Progress " };
const byte result_RandomInfinite[16]={ " Infinite Mode  " };
const byte result_RandomCleared[16]={ " Round Cleared  " };
const byte* const resultRandom_Headers[2]={result_RandomNormal,result_RandomInfinite};

// displayed at 0x228A
const byte result_RandomRecord[12]={ "\x8BNew Record!" };

// game over, man! (displayed inside result box; 20x10)
const byte result_GameOver[200]={
	0x91,0x86,0x87,0x91,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x91,0x86,0x87,0x91,
	0x91,0x84,0x85,0x91,0x70,0x1c,0x67,0x70,0x1c,0x71,0x70,0x75,0x71,0x70,0x1c,0x67,0x91,0x84,0x85,0x91,
	0x91,0x86,0x87,0x91,0x1d,0xcc,0x71,0x98,0x1c,0x9f,0x1d,0x1d,0x1d,0x98,0x67,0x00,0x91,0x86,0x87,0x91,
	0x91,0x84,0x85,0x91,0x1d,0x00,0x1d,0x1d,0x00,0x1d,0x1d,0x9d,0x1d,0x1d,0x00,0x00,0x91,0x84,0x85,0x91,
	0x91,0x86,0x87,0x91,0x72,0x1c,0x73,0xa4,0x00,0xa4,0xa4,0x00,0xa4,0x72,0x1c,0x67,0x91,0x86,0x87,0x91,
	0x91,0x84,0x85,0x91,0x70,0x1c,0x71,0xa3,0x00,0xa3,0x70,0x1c,0x67,0x70,0x1c,0x71,0x91,0x84,0x85,0x91,
	0x91,0x86,0x87,0x91,0x1d,0x00,0x1d,0x1d,0x00,0x1d,0x98,0x67,0x00,0x98,0x1c,0xaa,0x91,0x86,0x87,0x91,
	0x91,0x84,0x85,0x91,0x1d,0x00,0x1d,0x1d,0xa8,0xa9,0x1d,0x00,0x00,0x1d,0x00,0x1d,0x91,0x84,0x85,0x91,
	0x91,0x86,0x87,0x91,0x72,0x1c,0x73,0x72,0xa7,0x00,0x72,0x1c,0x67,0xa4,0x00,0xa4,0x91,0x86,0x87,0x91,
	0x91,0x84,0x85,0x91,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x91,0x84,0x85,0x91
};

/* Puzzle Data */
#define NUM_NORMAL_PUZZLES 20
/*
struct puzzleDataNormal {
	const byte name[NUM_NORMAL_PUZZLES];	// puzzle name (\xFF terminated)
	const byte data[NUM_NORMAL_PUZZLES];	// puzzle data
};
*/

// each puzzle size has a different number of bytes.
// these numbers are for the raw puzzle data (no header).
const byte puzzleSizeBytes[12]={ 4,5,7,8,11,13,16,18,21,25,29,32 };

#include "puzzles.h"

#endif
