-- Family Picross Level Decoder
-- usage: leveldisp.lua [path_tolevelBinfile]
--local bit = require("bit")
local bdc = require("BinDecHex")
local levelFile=...

local diffNames = {
	"easy", "medium", "hard", "expert", "master"
	-- everything else is invalid.
}

local function Usage() print("usage: leveldisp.lua [path_tolevelBinfile]") end

print("Family Picross Level Decoder tool")
if not levelFile then Usage() return end

-- fuckin' open the binary file (which only has level data, no names)
local file = io.open(levelFile,"rb")
if file == nil then
	print(string.format("Could not open '%s' for reading.",levelFile))
	return
end

print(string.format("Successfully opened '%s'.",levelFile))

-- read file
local fileData = file:read("*a")

if not fileData then
	-- there's no data
	print(string.format("Unable to find any data in '%s'.",levelFile))
	file:close();
	return
end

-- done with the file
file:close();

-- convert file contents into internal table
local bytes = {}
local displayBytes = {}
for b in string.gfind(fileData,".") do
	bytes[#bytes+1] = string.format("%x", string.byte(b))
	displayBytes[#displayBytes+1] = string.format("%02X", string.byte(b))
end

print("\nHex representation:")
local hexRep = ""
for i=1,#displayBytes do
	hexRep = hexRep.."0x"..displayBytes[i]
	if i < #displayBytes then
		hexRep = hexRep..","
	end
end
print(hexRep)
print("[Level Display]")

-- byte 1 is the size/difficulty
local levelInfo = bytes[1]
-- 00001111 - size (x0-xB matches up with 5-16; xC-xF are invalid)
local size = bdc.BMAnd(levelInfo,bdc.Bin2Hex("00001111"))
local realSize = bdc.Hex2Dec(size)+5
print(realSize)

-- 01110000 - difficulty
local difficulty = bdc.BMAnd(levelInfo,bdc.Bin2Hex("01110000"))
difficulty = bdc.BShRight(difficulty,4)
print(diffNames[tonumber(difficulty)+1])

-- the other bytes are the puzzle data itself.
-- we have the real size of the puzzle, which dictates the following:
-- 1) How many items to display in a line
-- 2) The format of the levels. (aka how the bytes map to the rows)

if realSize == 8 then
	-- easiest; every byte is one row!
	for i=2,#bytes do
		print(bdc.Hex2Bin(bytes[i]))
	end
elseif realSize == 16 then
	-- still easy; every two bytes are one row.
	for i=2,#bytes-1,2 do
		-- some surgery may need to be done if the width of the combined
		-- values is less than the realSize.
		--print(string.format("combining %d (%s) and %d (%s)...",i,bytes[i],i+1,bytes[i+1]))
		local rowA = bdc.Hex2Bin(bytes[i])
		local rowB = bdc.Hex2Bin(bytes[i+1])

		-- binary surgery
		if string.len(rowA) < 8 then
			local diffLen = 8-string.len(rowA)
			for i=1,diffLen do rowA = "0"..rowA end
		end
		if string.len(rowB) < 8 then
			local diffLen = 8-string.len(rowB)
			for i=1,diffLen do rowB = "0"..rowB end
		end

		local combined = rowA..rowB
		print(combined)
	end
elseif realSize == 12 then
	-- easier than the rest; there's a pattern to it.
else
	-- everything else needs to be hardcoded because I suck!
end
